@@ -33,7 +33,8 @@ _CODE_NAMES = {'2013.1': 'Grizzly',
                '2015.1': 'Kilo',
                '5.0.0': 'Liberty',
                '6.0.0': 'Mitaka',
-               '7.0.0': 'Newton'}
+               '7.0.0': 'Newton',
+               '8.0.0': 'Ocata'}
 
 all_resources = {}
 
