@@ -84,7 +84,9 @@ service_opts = [
                default=5,
                help=_('Maximum depth allowed when using nested stacks.')),
     cfg.IntOpt('num_engine_workers',
-               help=_('Number of heat-engine processes to fork and run.'))]
+               help=_('Number of heat-engine processes to fork and run. '
+                      'Will default to either to 4 or number of CPUs on '
+                      'the host, whichever is greater.'))]
 
 engine_opts = [
     cfg.ListOpt('plugin_dirs',
@@ -94,6 +96,9 @@ engine_opts = [
     cfg.StrOpt('environment_dir',
                default='/etc/heat/environment.d',
                help=_('The directory to search for environment files.')),
+    cfg.StrOpt('template_dir',
+               default='/etc/heat/templates',
+               help=_('The directory to search for template files.')),
     cfg.StrOpt('deferred_auth_method',
                choices=['password', 'trusts'],
                default='trusts',
@@ -132,30 +137,42 @@ engine_opts = [
                help=_('Number of times to retry when a client encounters an '
                       'expected intermittent error. Set to 0 to disable '
                       'retries.')),
+    # Server host name limit to 53 characters by due to typical default
+    # linux HOST_NAME_MAX of 64, minus the .novalocal appended to the name
+    cfg.IntOpt('max_server_name_length',
+               default=53,
+               max=53,
+               help=_('Maximum length of a server name to be used '
+                      'in nova.')),
     cfg.IntOpt('max_interface_check_attempts',
                min=1,
                default=10,
                help=_('Number of times to check whether an interface has '
                       'been attached or detached.')),
     cfg.IntOpt('event_purge_batch_size',
-               default=10,
+               min=1,
+               default=200,
                help=_("Controls how many events will be pruned whenever a "
-                      "stack's events exceed max_events_per_stack. Set this "
+                      "stack's events are purged. Set this "
                       "lower to keep more events at the expense of more "
                       "frequent purges.")),
     cfg.IntOpt('max_events_per_stack',
                default=1000,
-               help=_('Maximum events that will be available per stack. Older'
-                      ' events will be deleted when this is reached. Set to 0'
-                      ' for unlimited events per stack.')),
+               help=_('Rough number of maximum events that will be available '
+                      'per stack. Actual number of events can be a bit '
+                      'higher since purge checks take place randomly '
+                      '200/event_purge_batch_size percent of the time. '
+                      'Older events are deleted when events are purged. '
+                      'Set to 0 for unlimited events per stack.')),
     cfg.IntOpt('stack_action_timeout',
                default=3600,
                help=_('Timeout in seconds for stack action (ie. create or'
                       ' update).')),
     cfg.IntOpt('error_wait_time',
                default=240,
-               help=_('Error wait time in seconds for stack action (ie. create'
-                      ' or update).')),
+               help=_('The amount of time in seconds after an error has'
+                      ' occurred that tasks may continue to run before'
+                      ' being cancelled.')),
     cfg.IntOpt('engine_life_check_timeout',
                default=2,
                help=_('RPC timeout for the engine liveness check that is used'
@@ -170,7 +187,7 @@ engine_opts = [
                 default=False,
                 help=_('Enable the preview Stack Adopt feature.')),
     cfg.BoolOpt('convergence_engine',
-                default=False,
+                default=True,
                 help=_('Enables engine with convergence architecture. All '
                        'stacks with this option will be created using '
                        'convergence engine.')),
@@ -217,6 +234,19 @@ engine_opts = [
                       'credentials. ZAQAR_SIGNAL will create a dedicated '
                       'zaqar queue to be signaled using the provided keystone '
                       'credentials.')),
+    cfg.StrOpt('default_user_data_format',
+               choices=['HEAT_CFNTOOLS',
+                        'RAW',
+                        'SOFTWARE_CONFIG'],
+               default='HEAT_CFNTOOLS',
+               help=_('Template default for how the user_data should be '
+                      'formatted for the server. For HEAT_CFNTOOLS, the '
+                      'user_data is bundled as part of the heat-cfntools '
+                      'cloud-init boot configuration data. For RAW the '
+                      'user_data is passed to Nova unmodified. For '
+                      'SOFTWARE_CONFIG user_data is bundled as part of the '
+                      'software config data, and metadata is derived from any '
+                      'associated SoftwareDeployment resources.')),
     cfg.ListOpt('hidden_stack_tags',
                 default=['data-processing-cluster'],
                 help=_('Stacks containing these tag names will be hidden. '
@@ -237,11 +267,11 @@ engine_opts = [
                        ' set to the id of the resource\'s parent stack,'
                        ' heat_stack_name will be set to the name of the'
                        ' resource\'s parent stack, heat_path_in_stack will'
-                       ' be set to a list of tuples, (stackresourcename,'
-                       ' stackname) with list[0] being (None, rootstackname),'
-                       ' heat_resource_name will be set to the resource\'s'
-                       ' name, and heat_resource_uuid will be set to the'
-                       ' resource\'s orchestration id.')),
+                       ' be set to a list of comma delimited strings of'
+                       ' stackresourcename and stackname with list[0] being'
+                       ' \'rootstackname\', heat_resource_name will be set to'
+                       ' the resource\'s name, and heat_resource_uuid will be'
+                       ' set to the resource\'s orchestration id.')),
     cfg.BoolOpt('encrypt_parameters_and_properties',
                 default=False,
                 help=_('Encrypt template parameters that were marked as'
@@ -331,6 +361,14 @@ revision_opts = [
                       'separately, you can move this section to a different '
                       'file and add it as another config option.'))]
 
+volumes_group = cfg.OptGroup('volumes')
+volumes_opts = [
+    cfg.BoolOpt('backups_enabled',
+                default=True,
+                help=_("Indicate if cinder-backup service is enabled. "
+                       "This is a temporary workaround until cinder-backup "
+                       "service becomes discoverable, see LP#1334856."))]
+
 
 def startup_sanity_check():
     if (not cfg.CONF.stack_user_domain_id and
@@ -364,13 +402,14 @@ def list_opts():
     yield paste_deploy_group.name, paste_deploy_opts
     yield auth_password_group.name, auth_password_opts
     yield revision_group.name, revision_opts
+    yield volumes_group.name, volumes_opts
     yield profiler.list_opts()[0]
     yield 'clients', default_clients_opts
 
-    for client in ('barbican', 'ceilometer', 'cinder', 'designate', 'glance',
-                   'heat', 'keystone', 'magnum', 'manila', 'mistral',
-                   'neutron', 'nova', 'sahara', 'senlin', 'swift', 'trove',
-                   'zaqar'
+    for client in ('aodh', 'barbican', 'ceilometer', 'cinder', 'designate',
+                   'glance', 'heat', 'keystone', 'magnum', 'manila', 'mistral',
+                   'monasca', 'neutron', 'nova', 'sahara', 'senlin', 'swift',
+                   'trove', 'zaqar'
                    ):
         client_specific_group = 'clients_' + client
         yield client_specific_group, clients_opts
@@ -469,6 +508,21 @@ def get_client_option(client, option):
     return getattr(cfg.CONF.clients, option)
 
 
+def get_ssl_options(client):
+    # Look for the ssl options in the [clients_${client}] section
+    cacert = get_client_option(client, 'ca_file')
+    insecure = get_client_option(client, 'insecure')
+    cert = get_client_option(client, 'cert_file')
+    key = get_client_option(client, 'key_file')
+    if insecure:
+        verify = False
+    else:
+        verify = cacert or True
+    if cert and key:
+        cert = (cert, key)
+    return {'verify': verify, 'cert': cert}
+
+
 def set_config_defaults():
     """This method updates all configuration default values."""
     # CORS Defaults
