@@ -421,10 +421,12 @@ def format_notification_body(stack):
         rpc_api.NOTIFY_STATE: state,
         rpc_api.NOTIFY_STATE_REASON: stack.status_reason,
         rpc_api.NOTIFY_CREATE_AT: heat_timeutils.isotime(stack.created_time),
-        rpc_api.NOTIFY_DESCRIPTION: stack.t[stack.t.DESCRIPTION],
         rpc_api.NOTIFY_TAGS: stack.tags,
         rpc_api.NOTIFY_UPDATE_AT: updated_at
     }
+    if stack.t is not None:
+        result[rpc_api.NOTIFY_DESCRIPTION] = stack.t[stack.t.DESCRIPTION]
+
     return result
 
 
@@ -541,6 +543,13 @@ def format_validate_parameter(param):
             if c.max is not None:
                 res[rpc_api.PARAM_MAX_VALUE] = c.max
 
+        elif isinstance(c, constr.Modulo):
+            if c.step is not None:
+                res[rpc_api.PARAM_STEP] = c.step
+
+            if c.offset is not None:
+                res[rpc_api.PARAM_OFFSET] = c.offset
+
         elif isinstance(c, constr.AllowedValues):
             res[rpc_api.PARAM_ALLOWED_VALUES] = list(c.allowed)
 
@@ -560,7 +569,7 @@ def format_validate_parameter(param):
     return res
 
 
-def format_software_config(sc, detail=True):
+def format_software_config(sc, detail=True, include_project=False):
     if sc is None:
         return
     result = {
@@ -575,6 +584,9 @@ def format_software_config(sc, detail=True):
         result[rpc_api.SOFTWARE_CONFIG_INPUTS] = sc.config['inputs']
         result[rpc_api.SOFTWARE_CONFIG_OUTPUTS] = sc.config['outputs']
         result[rpc_api.SOFTWARE_CONFIG_OPTIONS] = sc.config['options']
+
+    if include_project:
+        result[rpc_api.SOFTWARE_CONFIG_PROJECT] = sc.tenant
     return result
 
 
