@@ -22,6 +22,7 @@ from stevedore import enabled
 from heat.common import exception
 from heat.common.i18n import _
 from heat.common.i18n import _LW
+from heat.common import pluginutils
 
 LOG = logging.getLogger(__name__)
 
@@ -31,7 +32,8 @@ _default_backend = "heat.engine.clients.OpenStackClients"
 cloud_opts = [
     cfg.StrOpt('cloud_backend',
                default=_default_backend,
-               help="Fully qualified class name to use as a client backend.")
+               help=_("Fully qualified class name to use as "
+                      "a client backend."))
 ]
 cfg.CONF.register_opts(cloud_opts)
 
@@ -64,10 +66,13 @@ class OpenStackClients(object):
             self._client_plugins[name] = client_plugin
             return client_plugin
 
-    def client(self, name):
+    def client(self, name, version=None):
         client_plugin = self.client_plugin(name)
         if client_plugin:
-            return client_plugin.client()
+            if version:
+                return client_plugin.client(version=version)
+            else:
+                return client_plugin.client()
 
         if name in self._clients:
             return self._clients[name]
@@ -80,14 +85,6 @@ class OpenStackClients(object):
             return client
         LOG.warning(_LW('Requested client "%s" not found'), name)
 
-    @property
-    def auth_token(self):
-        # Always use the auth_token from the keystone() client, as
-        # this may be refreshed if the context contains credentials
-        # which allow reissuing of a new token before the context
-        # auth_token expiry (e.g trust_id or username/password)
-        return self.client('keystone').auth_token
-
 
 class ClientBackend(object):
     """Class for delaying choosing the backend client module.
@@ -135,7 +132,8 @@ def initialise():
     _mgr = enabled.EnabledExtensionManager(
         namespace='heat.clients',
         check_func=client_is_available,
-        invoke_on_load=False)
+        invoke_on_load=False,
+        on_load_failure_callback=pluginutils.log_fail_msg)
 
 
 def list_opts():
