@@ -13,13 +13,14 @@
 
 from cinderclient import client as cc
 from cinderclient import exceptions
-from keystoneclient import exceptions as ks_exceptions
+from keystoneauth1 import exceptions as ks_exceptions
 from oslo_log import log as logging
 
 from heat.common import exception
 from heat.common.i18n import _
 from heat.common.i18n import _LI
 from heat.engine.clients import client_plugin
+from heat.engine.clients import os as os_client
 from heat.engine import constraints
 
 
@@ -32,64 +33,52 @@ class CinderClientPlugin(client_plugin.ClientPlugin):
 
     exceptions_module = exceptions
 
-    service_types = [VOLUME, VOLUME_V2] = ['volume', 'volumev2']
+    service_types = [VOLUME_V2, VOLUME_V3] = ['volumev2', 'volumev3']
 
     def get_volume_api_version(self):
         '''Returns the most recent API version.'''
-
-        endpoint_type = self._get_client_option(CLIENT_NAME, 'endpoint_type')
+        self.interface = self._get_client_option(CLIENT_NAME, 'endpoint_type')
         try:
-            self.url_for(service_type=self.VOLUME_V2,
-                         endpoint_type=endpoint_type)
-            return 2
+            self.context.keystone_session.get_endpoint(
+                service_type=self.VOLUME_V3,
+                interface=self.interface)
+            self.service_type = self.VOLUME_V3
+            self.client_version = '3'
         except ks_exceptions.EndpointNotFound:
             try:
-                self.url_for(service_type=self.VOLUME,
-                             endpoint_type=endpoint_type)
-                return 1
+                self.context.keystone_session.get_endpoint(
+                    service_type=self.VOLUME_V2,
+                    interface=self.interface)
+                self.service_type = self.VOLUME_V2
+                self.client_version = '2'
             except ks_exceptions.EndpointNotFound:
-                return None
+                raise exception.Error(_('No volume service available.'))
 
     def _create(self):
-
-        con = self.context
-
-        volume_api_version = self.get_volume_api_version()
-        if volume_api_version == 1:
-            service_type = self.VOLUME
-            client_version = '1'
-        elif volume_api_version == 2:
-            service_type = self.VOLUME_V2
-            client_version = '2'
-        else:
-            raise exception.Error(_('No volume service available.'))
-        LOG.info(_LI('Creating Cinder client with volume API version %d.'),
-                 volume_api_version)
-
-        endpoint_type = self._get_client_option(CLIENT_NAME, 'endpoint_type')
+        self.get_volume_api_version()
+        extensions = cc.discover_extensions(self.client_version)
         args = {
-            'service_type': service_type,
-            'auth_url': con.auth_url or '',
-            'project_id': con.tenant_id,
-            'username': None,
-            'api_key': None,
-            'endpoint_type': endpoint_type,
+            'session': self.context.keystone_session,
+            'extensions': extensions,
+            'interface': self.interface,
+            'service_type': self.service_type,
+            'region_name': self._get_region_name(),
             'http_log_debug': self._get_client_option(CLIENT_NAME,
-                                                      'http_log_debug'),
-            'cacert': self._get_client_option(CLIENT_NAME, 'ca_file'),
-            'insecure': self._get_client_option(CLIENT_NAME, 'insecure')
+                                                      'http_log_debug')
         }
 
-        client = cc.Client(client_version, **args)
-        management_url = self.url_for(service_type=service_type,
-                                      endpoint_type=endpoint_type)
-        client.client.auth_token = self.auth_token
-        client.client.management_url = management_url
-
-        client.volume_api_version = volume_api_version
-
+        client = cc.Client(self.client_version, **args)
         return client
 
+    @os_client.MEMOIZE_EXTENSIONS
+    def _list_extensions(self):
+        extensions = self.client().list_extensions.show_all()
+        return set(extension.alias for extension in extensions)
+
+    def has_extension(self, alias):
+        """Check if specific extension is present."""
+        return alias in self._list_extensions()
+
     def get_volume(self, volume):
         try:
             return self.client().volumes.get(volume)
@@ -114,10 +103,7 @@ class CinderClientPlugin(client_plugin.ClientPlugin):
         vt_id = None
         volume_type_list = self.client().volume_types.list()
         for vt in volume_type_list:
-            if vt.name == volume_type:
-                vt_id = vt.id
-                break
-            if vt.id == volume_type:
+            if volume_type in [vt.name, vt.id]:
                 vt_id = vt.id
                 break
         if vt_id is None:
@@ -126,6 +112,13 @@ class CinderClientPlugin(client_plugin.ClientPlugin):
 
         return vt_id
 
+    def get_qos_specs(self, qos_specs):
+        try:
+            qos = self.client().qos_specs.get(qos_specs)
+        except exceptions.NotFound:
+            qos = self.client().qos_specs.find(name=qos_specs)
+        return qos.id
+
     def is_not_found(self, ex):
         return isinstance(ex, exceptions.NotFound)
 
@@ -203,3 +196,10 @@ class VolumeTypeConstraint(BaseCinderConstraint):
 class VolumeBackupConstraint(BaseCinderConstraint):
 
     resource_getter_name = 'get_volume_backup'
+
+
+class QoSSpecsConstraint(BaseCinderConstraint):
+
+    expected_exceptions = (exceptions.NotFound,)
+
+    resource_getter_name = 'get_qos_specs'
