@@ -42,7 +42,8 @@ _default_keystone_backend = (
 keystone_opts = [
     cfg.StrOpt('keystone_backend',
                default=_default_keystone_backend,
-               help="Fully qualified class name to use as a keystone backend.")
+               help=_("Fully qualified class name to use as a "
+                      "keystone backend."))
 ]
 cfg.CONF.register_opts(keystone_opts)
 
@@ -235,7 +236,7 @@ class KsClientWrapper(object):
         """Delete the specified trust."""
         try:
             self.client.trusts.delete(trust_id)
-        except (ks_exception.NotFound, ks_exception.Unauthorized):
+        except ks_exception.NotFound:
             pass
 
     def _get_username(self, username):
