@@ -11,45 +11,36 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_utils import importutils
+from monascaclient import client
+from monascaclient import exc as monasca_exc
 
 from heat.common import exception as heat_exc
 from heat.engine.clients import client_plugin
 from heat.engine import constraints
 
-client = importutils.try_import('monascaclient.client')
-monasca_exc = importutils.try_import('monascaclient.exc')
-
 CLIENT_NAME = 'monasca'
 
 
 class MonascaClientPlugin(client_plugin.ClientPlugin):
     exceptions_module = [monasca_exc]
+
     service_types = [MONITORING] = ['monitoring']
 
     VERSION = '2_0'
 
-    @staticmethod
-    def is_available():
-        return client is not None
-
     def _create(self):
-        args = self._get_client_args(service_name=CLIENT_NAME,
-                                     service_type=self.MONITORING)
-
+        interface = self._get_client_option(CLIENT_NAME, 'endpoint_type')
+        endpoint = self.url_for(service_type=self.MONITORING,
+                                endpoint_type=interface)
+        # Change this to use session once it's supported by monascaclient
         return client.Client(
             self.VERSION,
-            endpoint=args['os_endpoint'],
-            endpoint_type=args['endpoint_type'],
-            auth_url=args['auth_url'],
-            token=args['token'](),
-            project_id=args['project_id'],
-            service_type=args['service_type'],
-            os_cacert=args['cacert'],
-            cert_file=args['cert_file'],
-            key_file=args['key_file'],
-            insecure=args['insecure']
-        )
+            token=self.context.keystone_session.get_token(),
+            endpoint=endpoint,
+            cacert=self._get_client_option(CLIENT_NAME, 'ca_file'),
+            cert_file=self._get_client_option(CLIENT_NAME, 'cert_file'),
+            key_file=self._get_client_option(CLIENT_NAME, 'key_file'),
+            insecure=self._get_client_option(CLIENT_NAME, 'insecure'))
 
     def is_not_found(self, ex):
         return isinstance(ex, monasca_exc.NotFound)
