@@ -169,8 +169,11 @@ class GetAttThenSelect(function.Function):
 
         return self.args[0], self.args[1], self.args[2:]
 
+    def _res_name(self):
+        return function.resolve(self._resource_name)
+
     def _resource(self, path='unknown'):
-        resource_name = function.resolve(self._resource_name)
+        resource_name = self._res_name()
 
         try:
             return self.stack[resource_name]
@@ -179,7 +182,7 @@ class GetAttThenSelect(function.Function):
                                                      key=path)
 
     def dep_attrs(self, resource_name):
-        if self._resource().name == resource_name:
+        if self._res_name() == resource_name:
             attrs = [function.resolve(self._attribute)]
         else:
             attrs = []
@@ -263,7 +266,7 @@ class GetAtt(GetAttThenSelect):
             return None
 
     def dep_attrs(self, resource_name):
-        if self._resource().name == resource_name:
+        if self._res_name() == resource_name:
             path = function.resolve(self._path_components)
             attr = [function.resolve(self._attribute)]
             if path:
@@ -311,7 +314,7 @@ class GetAttAllAttributes(GetAtt):
         """Check if there is no attribute_name defined, return empty chain."""
         if self._attribute is not None:
             return super(GetAttAllAttributes, self).dep_attrs(resource_name)
-        elif self._resource().name == resource_name:
+        elif self._res_name() == resource_name:
             res = self._resource()
             attrs = six.iterkeys(res.attributes_schema)
         else:
@@ -355,6 +358,8 @@ class Replace(function.Function):
     of equal length, lexicographically smaller keys are preferred.
     """
 
+    _strict = False
+
     def __init__(self, stack, fn_name, args):
         super(Replace, self).__init__(stack, fn_name, args)
 
@@ -373,13 +378,13 @@ class Replace(function.Function):
             mapping = self.args['params']
             string = self.args['template']
         except (KeyError, TypeError):
-            example = ('''str_replace:
+            example = _('''%s:
               template: This is var1 template var2
               params:
                 var1: a
-                var2: string''')
-            raise KeyError(_('"str_replace" syntax should be %s') %
-                           example)
+                var2: string''') % self.fn_name
+            raise KeyError(_('"%(fn_name)s" syntax should be %(example)s') %
+                           {'fn_name': self.fn_name, 'example': example})
         else:
             return mapping, string
 
@@ -399,6 +404,9 @@ class Replace(function.Function):
         template = function.resolve(self._string)
         mapping = function.resolve(self._mapping)
 
+        if self._strict:
+            unreplaced_keys = set(mapping)
+
         if not isinstance(template, six.string_types):
             raise TypeError(_('"%s" template must be a string') % self.fn_name)
 
@@ -416,11 +424,24 @@ class Replace(function.Function):
 
             remaining_keys = keys[1:]
             value = self._validate_replacement(mapping[placeholder])
-            return [value.join(replace(s.split(placeholder),
+
+            def string_split(s):
+                ss = s.split(placeholder)
+                if self._strict and len(ss) > 1:
+                    unreplaced_keys.discard(placeholder)
+                return ss
+
+            return [value.join(replace(string_split(s),
                                        remaining_keys)) for s in strings]
 
-        return replace([template], sorted(sorted(mapping),
-                                          key=len, reverse=True))[0]
+        ret_val = replace([template], sorted(sorted(mapping),
+                                             key=len, reverse=True))[0]
+        if self._strict and len(unreplaced_keys) > 0:
+            raise ValueError(
+                _("The following params were not found in the template: %s") %
+                ','.join(sorted(sorted(unreplaced_keys),
+                                key=len, reverse=True)))
+        return ret_val
 
 
 class ReplaceJson(Replace):
@@ -454,7 +475,7 @@ class ReplaceJson(Replace):
                                   float, bool)):
             if isinstance(value, (collections.Mapping, collections.Sequence)):
                 try:
-                    return jsonutils.dumps(value, default=None)
+                    return jsonutils.dumps(value, default=None, sort_keys=True)
                 except TypeError:
                     raise TypeError(_('"%(name)s" params must be strings, '
                                       'numbers, list or map. '
@@ -468,6 +489,17 @@ class ReplaceJson(Replace):
         return six.text_type(value)
 
 
+class ReplaceJsonStrict(ReplaceJson):
+    """A function for performing string substituions.
+
+    str_replace_strict is identical to the str_replace function, only
+    a ValueError is raised if any of the params are not present in
+    the template.
+    """
+
+    _strict = True
+
+
 class GetFile(function.Function):
     """A function for including a file inline.
 
@@ -622,7 +654,7 @@ class JoinMultiple(function.Function):
                 return s
             elif isinstance(s, (collections.Mapping, collections.Sequence)):
                 try:
-                    return jsonutils.dumps(s, default=None)
+                    return jsonutils.dumps(s, default=None, sort_keys=True)
                 except TypeError:
                     msg = _('Items to join must be string, map or list. '
                             '%s failed json serialization'
@@ -878,15 +910,19 @@ class Repeat(function.Function):
 
     def result(self):
         for_each = function.resolve(self._for_each)
-        if not all(self._valid_arg(l) for l in for_each.values()):
+        keys, lists = six.moves.zip(*for_each.items())
+
+        # use empty list for references(None) else validation will fail
+        values = [[] if value is None else value for value in lists]
+
+        if not all(self._valid_arg(l) for l in values):
             raise TypeError(_('The values of the "for_each" argument to '
                               '"%s" must be lists') % self.fn_name)
 
         template = function.resolve(self._template)
 
-        keys, lists = six.moves.zip(*for_each.items())
         return [self._do_replacement(keys, replacements, template)
-                for replacements in itertools.product(*lists)]
+                for replacements in itertools.product(*values)]
 
 
 class RepeatWithMap(Repeat):
@@ -1238,3 +1274,49 @@ class Or(ConditionBoolean):
     def result(self):
         return any(self._get_condition(cd)
                    for cd in function.resolve(self.args))
+
+
+class Filter(function.Function):
+    """A function for filtering out values from lists.
+
+    Takes the form::
+
+        filter:
+          - <values>
+          - <list>
+
+    Returns a new list without the values.
+    """
+    def __init__(self, stack, fn_name, args):
+        super(Filter, self).__init__(stack, fn_name, args)
+
+        self._values, self._sequence = self._parse_args()
+
+    def _parse_args(self):
+        if (not isinstance(self.args, collections.Sequence) or
+                isinstance(self.args, six.string_types)):
+            raise TypeError(_('Argument to "%s" must be a list') %
+                            self.fn_name)
+
+        if len(self.args) != 2:
+            raise ValueError(_('"%(fn)s" expected 2 arguments of the form '
+                               '[values, sequence] but got %(len)d arguments '
+                               'instead') %
+                             {'fn': self.fn_name, 'len': len(self.args)})
+
+        return self.args[0], self.args[1]
+
+    def result(self):
+        sequence = function.resolve(self._sequence)
+        if not sequence:
+            return sequence
+        if not isinstance(sequence, list):
+            raise TypeError(_('"%s" only works with lists') % self.fn_name)
+
+        values = function.resolve(self._values)
+        if not values:
+            return sequence
+        if not isinstance(values, list):
+            raise TypeError(
+                _('"%(fn)s" filters a list of values') % self.fn_name)
+        return [i for i in sequence if i not in values]
