@@ -14,7 +14,6 @@
 import base64
 import contextlib
 import datetime as dt
-import warnings
 import weakref
 
 from oslo_config import cfg
@@ -45,6 +44,7 @@ from heat.engine import scheduler
 from heat.engine import support
 from heat.objects import resource as resource_objects
 from heat.objects import resource_data as resource_data_objects
+from heat.objects import resource_properties_data as rpd_objects
 from heat.objects import stack as stack_objects
 from heat.rpc import client as rpc_client
 
@@ -72,7 +72,11 @@ class NoActionRequired(Exception):
     Resource subclasses should raise this exception from handle_signal() to
     suppress recording of an event corresponding to the signal.
     """
-    pass
+    def __init__(self, res_name='Unknown', reason=''):
+        msg = (_("The resource %(res)s could not perform "
+                 "scaling action: %(reason)s") %
+               {'res': res_name, 'reason': reason})
+        super(Exception, self).__init__(six.text_type(msg))
 
 
 class PollDelay(Exception):
@@ -229,6 +233,8 @@ class Resource(object):
                      client_resolve=False)
         self.update_policy = self.t.update_policy(self.update_policy_schema,
                                                   self.context)
+        self._update_allowed_properties = self.calc_update_allowed(
+            self.properties)
         self.attributes_schema.update(self.base_attributes_schema)
         self.attributes = self._init_attributes()
 
@@ -248,6 +254,7 @@ class Resource(object):
         self.uuid = None
         self._data = None
         self._rsrc_metadata = None
+        self._rsrc_prop_data = None
         self._stored_properties_data = None
         self.created_time = stack.created_time
         self.updated_time = stack.updated_time
@@ -290,6 +297,7 @@ class Resource(object):
             self._data = {}
         self._rsrc_metadata = resource.rsrc_metadata
         self._stored_properties_data = resource.properties_data
+        self._rsrc_prop_data = resource.rsrc_prop_data
         self.created_time = resource.created_at
         self.updated_time = resource.updated_at
         self.needed_by = resource.needed_by
@@ -299,6 +307,10 @@ class Resource(object):
         self.current_template_id = resource.current_template_id
         self.root_stack_id = resource.root_stack_id
 
+    @property
+    def external_id(self):
+        return self.t.external_id()
+
     @property
     def stack(self):
         stack = self._stackref()
@@ -345,7 +357,7 @@ class Resource(object):
         # Don't set physical_resource_id so that a create is triggered.
         rs = {'stack_id': self.stack.id,
               'name': self.name,
-              'properties_data': self._stored_properties_data,
+              'rsrc_prop_data_id': self._create_or_replace_rsrc_prop_data(),
               'needed_by': self.needed_by,
               'requires': self.requires,
               'replaces': self.id,
@@ -375,6 +387,13 @@ class Resource(object):
         if translate:
             self.translate_properties(self.properties, client_resolve)
 
+    def calc_update_allowed(self, props):
+        update_allowed_set = set(self.update_allowed_properties)
+        for (psk, psv) in six.iteritems(props.props):
+            if psv.update_allowed():
+                update_allowed_set.add(psk)
+        return update_allowed_set
+
     def __eq__(self, other):
         """Allow == comparison of two resources."""
         # For the purposes of comparison, we declare two resource objects
@@ -460,13 +479,13 @@ class Resource(object):
     def _break_if_required(self, action, hook):
         """Block the resource until the hook is cleared if there is one."""
         if self.stack.env.registry.matches_hook(self.name, hook):
+            self.trigger_hook(hook)
             self._add_event(self.action, self.status,
                             _("%(a)s paused until Hook %(h)s is cleared")
                             % {'a': action, 'h': hook})
-            self.trigger_hook(hook)
-            LOG.info(_LI('Reached hook on %s'), six.text_type(self))
+            LOG.info(_LI('Reached hook on %s'), self)
 
-            while self.has_hook(hook):
+            while self.has_hook(hook) and self.status != self.FAILED:
                 try:
                     yield
                 except BaseException as exc:
@@ -525,23 +544,6 @@ class Resource(object):
         return identifier.ResourceIdentifier(resource_name=self.name,
                                              **self.stack.identifier())
 
-    def parsed_template(self, section=None, default=None):
-        """Return the parsed template data for the resource.
-
-        May be limited to only one section of the data, in which case a default
-        value may also be supplied.
-        """
-        warnings.warn('Resource.parsed_template() is deprecated and will be '
-                      'removed in the Ocata release. Use the '
-                      'ResourceDefinition API instead.',
-                      DeprecationWarning)
-
-        frozen = self.t.freeze()
-        if section is None:
-            return frozen
-
-        return frozen.get(section, default or {})
-
     def frozen_definition(self):
         if self._stored_properties_data is not None:
             args = {'properties': self._stored_properties_data}
@@ -565,11 +567,9 @@ class Resource(object):
         If any properties have changed which are not in
         update_allowed_properties, raises UpdateReplace.
         """
-        update_allowed_set = set(self.update_allowed_properties)
+        update_allowed_set = self.calc_update_allowed(after_props)
         immutable_set = set()
         for (psk, psv) in six.iteritems(after_props.props):
-            if psv.update_allowed():
-                update_allowed_set.add(psk)
             if psv.immutable():
                 immutable_set.add(psk)
 
@@ -662,18 +662,11 @@ class Resource(object):
         directly.
         """
         try:
-            reqd_by = self.stack.dependencies.required_by(self)
+            return [r.name for r in self.stack.dependencies.required_by(self)]
         except KeyError:
-            if self.stack.convergence:
-                # for convergence, fall back to building from needed_by
-                needed_by_ids = self.needed_by or set()
-                reqd_by = [r for r in self.stack.resources.values()
-                           if r.id in needed_by_ids]
-            else:
-                LOG.error('Getting required_by list for Resource not in '
-                          'dependency graph.')
-                return []
-        return [r.name for r in reqd_by]
+            # for convergence, fall back to building from needed_by
+            return [r.name for r in self.stack.resources.values()
+                    if r.id in self.needed_by]
 
     def client(self, name=None, version=None):
         client_name = name or self.default_client_name
@@ -872,7 +865,10 @@ class Resource(object):
             yield self.action_handler_task(action, args=handler_args)
 
     def _update_stored_properties(self):
+        old_props = self._stored_properties_data
         self._stored_properties_data = function.resolve(self.properties.data)
+        if self._stored_properties_data != old_props:
+            self._rsrc_prop_data = None
 
     def preview(self):
         """Default implementation of Resource.preview.
@@ -899,19 +895,18 @@ class Resource(object):
 
             runner(timeout=timeout, progress_callback=progress_callback)
 
-    def _validate_external_resource(self, external_id):
-        if self.entity:
+    def validate_external(self):
+        if self.external_id is not None:
             try:
-                self.resource_id = external_id
+                self.resource_id = self.external_id
                 self._show_resource()
             except Exception as ex:
-                LOG.debug("%s", ex)
                 if self.client_plugin().is_not_found(ex):
-                    error_message = _("Invalid external resource: Resource "
-                                      "%(external_id)s not found in "
-                                      "%(entity)s.") % {
-                                          'external_id': external_id,
-                                          'entity': self.entity}
+                    error_message = (_("Invalid external resource: Resource "
+                                       "%(external_id)s (%(type)s) can not "
+                                       "be found.") %
+                                     {'external_id': self.external_id,
+                                      'type': self.type()})
                     raise exception.StackValidationFailed(
                         message="%s" % error_message)
                 raise
@@ -923,21 +918,19 @@ class Resource(object):
         Subclasses should provide a handle_create() method to customise
         creation.
         """
-        external = self.t.external_id()
-        if external is not None:
-            self._validate_external_resource(external_id=external)
-
-            yield self._do_action(self.ADOPT,
-                                  resource_data={'resource_id': external})
-            self.check()
-            return
-
         action = self.CREATE
         if (self.action, self.status) != (self.INIT, self.COMPLETE):
             exc = exception.Error(_('State %s invalid for create')
                                   % six.text_type(self.state))
             raise exception.ResourceFailure(exc, self, action)
 
+        if self.external_id is not None:
+            yield self._do_action(self.ADOPT,
+                                  resource_data={
+                                      'resource_id': self.external_id})
+            self.check()
+            return
+
         # This method can be called when we replace a resource, too. In that
         # case, a hook has already been dealt with in `Resource.update` so we
         # shouldn't do it here again:
@@ -945,7 +938,7 @@ class Resource(object):
             yield self._break_if_required(
                 self.CREATE, environment.HOOK_PRE_CREATE)
 
-        LOG.info(_LI('creating %s'), six.text_type(self))
+        LOG.info(_LI('creating %s'), self)
 
         # Re-resolve the template, since if the resource Ref's
         # the StackId pseudo parameter, it will change after
@@ -1056,7 +1049,17 @@ class Resource(object):
         """
         rules = self.translation_rules(properties) or []
         for rule in rules:
-            rule.execute_rule(client_resolve)
+            try:
+                rule.execute_rule(client_resolve)
+            except exception.ResourcePropertyConflict as ex:
+                path = [self.stack.t.RESOURCES, self.name,
+                        self.stack.t.get_section_name(
+                            self.stack.t.RES_PROPERTIES)]
+                raise exception.StackValidationFailed(
+                    error='Property error',
+                    path=path,
+                    message=ex.message
+                )
 
     def cancel_grace_period(self):
         if self.status != self.IN_PROGRESS:
@@ -1091,14 +1094,22 @@ class Resource(object):
         """Needs replace based on tmpl_diff."""
         return False
 
+    def needs_replace_failed(self):
+        """Needs replace if resource is in *_FAILED."""
+        return True
+
     def _needs_update(self, after, before, after_props, before_props,
                       prev_resource, check_init_complete=True):
-        if self.status == self.FAILED or (self.stack.convergence and (
-                self.action, self.status) == (self.DELETE, self.COMPLETE)):
+        if self.status == self.FAILED:
+            # always replace when a resource is in CHECK_FAILED
+            if self.action == self.CHECK or self.needs_replace_failed():
+                raise UpdateReplace(self)
+
+        if self.state == (self.DELETE, self.COMPLETE):
             raise UpdateReplace(self)
 
-        if check_init_complete and (self.action == self.INIT
-                                    and self.status == self.COMPLETE):
+        if (check_init_complete and
+                self.state == (self.INIT, self.COMPLETE)):
             raise UpdateReplace(self)
 
         if self.needs_replace(after_props):
@@ -1172,7 +1183,7 @@ class Resource(object):
             try:
                 runner(timeout=timeout, progress_callback=progress_callback)
                 update_tmpl_id_and_requires()
-            except exception.UpdateReplace:
+            except UpdateReplace:
                 raise
             except BaseException:
                 with excutils.save_and_reraise_exception():
@@ -1244,14 +1255,14 @@ class Resource(object):
             except Exception as ex:
                 LOG.warning(_LW("Resource cannot be updated with it's "
                                 "live state in case of next "
-                                "error: %s"), six.text_type(ex))
+                                "error: %s"), ex)
         return after_props, before_props
 
     def _prepare_update_replace(self, action):
         try:
             if (self.stack.action == 'ROLLBACK' and
                     self.stack.status == 'IN_PROGRESS' and
-                    not cfg.CONF.convergence_engine):
+                    not self.stack.convergence):
                 # handle case, when it's rollback and we should restore
                 # old resource
                 self.restore_prev_rsrc()
@@ -1285,14 +1296,14 @@ class Resource(object):
         if before is None:
             before = self.frozen_definition()
 
-        external = after.external_id()
-        if before.external_id() != external:
+        after_external_id = after.external_id()
+        if self.external_id != after_external_id:
             msg = _("Update to property %(prop)s of %(name)s (%(res)s)"
                     ) % {'prop': hot_tmpl.HOTemplate20161014.RES_EXTERNAL_ID,
                          'res': self.type(), 'name': self.name}
             exc = exception.NotSupported(feature=msg)
             raise exception.ResourceFailure(exc, self, action)
-        elif external is not None:
+        elif after_external_id is not None:
             LOG.debug("Skip update on external resource.")
             return
 
@@ -1317,7 +1328,7 @@ class Resource(object):
                                           prev_resource):
                     return
 
-            if not cfg.CONF.convergence_engine:
+            if not self.stack.convergence:
                 if (self.action, self.status) in (
                         (self.CREATE, self.IN_PROGRESS),
                         (self.UPDATE, self.IN_PROGRESS),
@@ -1325,7 +1336,7 @@ class Resource(object):
                     exc = Exception(_('Resource update already requested'))
                     raise exception.ResourceFailure(exc, self, action)
 
-            LOG.info(_LI('updating %s'), six.text_type(self))
+            LOG.info(_LI('updating %s'), self)
 
             self.updated_time = datetime.utcnow()
 
@@ -1387,7 +1398,7 @@ class Resource(object):
         original state with the added message that check was not performed.
         """
         action = self.CHECK
-        LOG.info(_LI('Checking %s'), six.text_type(self))
+        LOG.info(_LI('Checking %s'), self)
 
         if hasattr(self, 'handle_%s' % action.lower()):
             return self._do_action(action)
@@ -1428,7 +1439,7 @@ class Resource(object):
                                   % six.text_type(self.state))
             raise exception.ResourceFailure(exc, self, action)
 
-        LOG.info(_LI('suspending %s'), six.text_type(self))
+        LOG.info(_LI('suspending %s'), self)
         return self._do_action(action)
 
     def resume(self):
@@ -1447,12 +1458,12 @@ class Resource(object):
             exc = exception.Error(_('State %s invalid for resume')
                                   % six.text_type(self.state))
             raise exception.ResourceFailure(exc, self, action)
-        LOG.info(_LI('resuming %s'), six.text_type(self))
+        LOG.info(_LI('resuming %s'), self)
         return self._do_action(action)
 
     def snapshot(self):
         """Snapshot the resource and return the created data, if any."""
-        LOG.info(_LI('snapshotting %s'), six.text_type(self))
+        LOG.info(_LI('snapshotting %s'), self)
         return self._do_action(self.SNAPSHOT)
 
     @scheduler.wrappertask
@@ -1502,7 +1513,7 @@ class Resource(object):
         This may be overridden by resource plugins to add extra
         validation logic specific to the resource implementation.
         """
-        LOG.info(_LI('Validating %s'), six.text_type(self))
+        LOG.info(_LI('Validating %s'), self)
         return self.validate_template()
 
     def validate_template(self):
@@ -1518,8 +1529,8 @@ class Resource(object):
                 self.stack.context,
                 self.t.resource_type
             )
-
-        function.validate(self.t)
+        path = '.'.join([self.stack.t.RESOURCES, self.name])
+        function.validate(self.t, path)
         self.validate_deletion_policy(self.t.deletion_policy())
         self.t.update_policy(self.update_policy_schema,
                              self.context).validate()
@@ -1649,7 +1660,7 @@ class Resource(object):
             yield self._break_if_required(
                 self.DELETE, environment.HOOK_PRE_DELETE)
 
-        LOG.info(_LI('deleting %s'), six.text_type(self))
+        LOG.info(_LI('deleting %s'), self)
 
         if self._stored_properties_data is not None:
             # On delete we can't rely on re-resolving the properties
@@ -1723,9 +1734,6 @@ class Resource(object):
 
         If self.id is set, we update the existing stack.
         """
-        properties_data_encrypted, properties_data = (
-            resource_objects.Resource.encrypt_properties_data(
-                self._stored_properties_data))
         if not self.root_stack_id:
             self.root_stack_id = self.stack.root_stack_id()
 
@@ -1735,6 +1743,8 @@ class Resource(object):
               'stack_id': self.stack.id,
               'physical_resource_id': self.resource_id,
               'name': self.name,
+              'rsrc_prop_data_id':
+                  self._create_or_replace_rsrc_prop_data(),
               'needed_by': self.needed_by,
               'requires': self.requires,
               'replaces': self.replaces,
@@ -1742,8 +1752,7 @@ class Resource(object):
               'current_template_id': self.current_template_id,
               'root_stack_id': self.root_stack_id,
               'updated_at': self.updated_time,
-              'properties_data': properties_data,
-              'properties_data_encrypted': properties_data_encrypted}
+              'properties_data': None}
 
         if set_metadata:
             metadata = self.t.metadata()
@@ -1763,7 +1772,7 @@ class Resource(object):
         """Add a state change event to the database."""
         physical_res_id = self.resource_id or self.physical_resource_name()
         ev = event.Event(self.context, self.stack, action, status, reason,
-                         physical_res_id, self.properties,
+                         physical_res_id, self._rsrc_prop_data,
                          self.name, self.type())
 
         ev.store()
@@ -1868,7 +1877,10 @@ class Resource(object):
             try:
                 obj = getattr(self.client(), self.entity)
                 resource = obj.get(self.resource_id)
-                return resource.to_dict()
+                if isinstance(resource, dict):
+                    return resource
+                else:
+                    return resource.to_dict()
             except AttributeError as ex:
                 LOG.warning(_LW("Resolving 'show' attribute has failed : %s"),
                             ex)
@@ -1897,7 +1909,11 @@ class Resource(object):
         :param resource_properties: properties of stored resource plugin.
         :param resource_data: data from current live state of a resource.
         """
-        return {}
+        resource_result = {}
+        for key in self._update_allowed_properties:
+            resource_result[key] = resource_data.get(key)
+
+        return resource_result
 
     def get_live_state(self, resource_properties):
         """Default implementation; should be overridden by resources.
@@ -2121,7 +2137,8 @@ class Resource(object):
             pass
         except Exception as ex:
             LOG.info(_LI('signal %(name)s : %(msg)s'),
-                     {'name': six.text_type(self), 'msg': ex},
+                     {'name': six.text_type(self),
+                      'msg': six.text_type(ex)},
                      exc_info=True)
             failure = exception.ResourceFailure(ex, self)
             raise failure
@@ -2245,6 +2262,18 @@ class Resource(object):
             self._data = None
             return True
 
+    def _create_or_replace_rsrc_prop_data(self):
+        if self._rsrc_prop_data is not None:
+            return self._rsrc_prop_data.id
+
+        if not self._stored_properties_data:
+            return None
+
+        self._rsrc_prop_data = \
+            rpd_objects.ResourcePropertiesData(self.context).create(
+                self.context, self._stored_properties_data)
+        return self._rsrc_prop_data.id
+
     def is_using_neutron(self):
         try:
             sess_client = self.client('neutron').httpclient
