@@ -13,9 +13,12 @@
 
 import copy
 
+from oslo_config import cfg
 from oslo_log import log as logging
 import six
 
+cfg.CONF.import_opt('max_server_name_length', 'heat.common.config')
+
 from heat.common import exception
 from heat.common.i18n import _
 from heat.common.i18n import _LI
@@ -340,9 +343,7 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         ),
     }
 
-    # Server host name limit to 53 characters by due to typical default
-    # linux HOST_NAME_MAX of 64, minus the .novalocal appended to the name
-    physical_resource_name_limit = 53
+    physical_resource_name_limit = cfg.CONF.max_server_name_length
 
     default_client_name = 'nova'
 
@@ -381,13 +382,12 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         if availability_zone is None:
             try:
                 server = self.client().servers.get(self.resource_id)
-                availability_zone = getattr(server,
-                                            'OS-EXT-AZ:availability_zone')
             except Exception as e:
                 self.client_plugin().ignore_not_found(e)
                 return
-
-        return availability_zone
+        # Default to None if Nova's
+        # OS-EXT-AZ:availability_zone extension is disabled
+        return getattr(server, 'OS-EXT-AZ:availability_zone', None)
 
     def _resolve_attribute(self, name):
         res = None
@@ -730,8 +730,8 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         return updaters
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        if 'Metadata' in tmpl_diff:
-            self.metadata_set(tmpl_diff['Metadata'])
+        if tmpl_diff.metadata_changed():
+            self.metadata_set(json_snippet.metadata())
         updaters = []
         server = None
         if self.TAGS in prop_diff:
