@@ -117,6 +117,7 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             properties.Schema.MAP,
             _('Key/value pairs to associate with the volume.'),
             update_allowed=True,
+            default={}
         ),
         IMAGE_REF: properties.Schema(
             properties.Schema.STRING,
@@ -281,6 +282,8 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
         return arguments
 
     def _resolve_attribute(self, name):
+        if self.resource_id is None:
+            return
         cinder = self.client()
         vol = cinder.volumes.get(self.resource_id)
         if name == self.METADATA_ATTR:
@@ -386,6 +389,16 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
         LOG.info(_LI('Volume %(id)s backup restore complete'), {'id': vol.id})
         return True
 
+    def needs_replace_failed(self):
+        if not self.resource_id:
+            return True
+
+        with self.client_plugin().ignore_not_found:
+            vol = self.client().volumes.get(self.resource_id)
+            return vol.status in ('error', 'deleting')
+
+        return True
+
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         vol = None
         cinder = self.client()
@@ -427,12 +440,9 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             prg_detach, prg_attach = self._detach_attach_progress(vol)
         # restore the volume from backup
         if self.BACKUP_ID in prop_diff:
-            if not vol:
-                vol = cinder.volumes.get(self.resource_id)
             prg_restore = progress.VolumeBackupRestoreProgress(
                 vol_id=self.resource_id,
                 backup_id=prop_diff.get(self.BACKUP_ID))
-            prg_detach, prg_attach = self._detach_attach_progress(vol)
         # extend volume size
         if self.SIZE in prop_diff:
             if not vol:
@@ -498,11 +508,6 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
 
     def check_update_complete(self, checkers):
         prg_restore, prg_detach, prg_resize, prg_access, prg_attach = checkers
-        # detach volume
-        if prg_detach:
-            if not prg_detach.nova_complete:
-                self._detach_volume_to_complete(prg_detach)
-                return False
         if prg_restore:
             if not prg_restore.called:
                 prg_restore.called = self._backup_restore(
@@ -512,6 +517,13 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             if not prg_restore.complete:
                 prg_restore.complete = self._check_backup_restore_complete()
                 return prg_restore.complete and not prg_resize
+        if not prg_resize and not prg_access:
+            return True
+        # detach volume
+        if prg_detach:
+            if not prg_detach.nova_complete:
+                self._detach_volume_to_complete(prg_detach)
+                return False
         # resize volume
         if prg_resize:
             if not prg_resize.called:
@@ -532,7 +544,7 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
         return True
 
     def handle_snapshot(self):
-        backup = self.client().backups.create(self.resource_id, force=True)
+        backup = self.client().backups.create(self.resource_id)
         self.data_set('backup_id', backup.id)
         return backup.id
 
@@ -635,7 +647,7 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             self.IMAGE_REF, self.IMAGE, self.SOURCE_VOLID)
         props = dict(
             (key, value) for (key, value) in
-            self.properties.data.items()
+            six.iteritems(defn.properties(self.properties_schema))
             if key not in ignore_props and value is not None)
         props[self.BACKUP_ID] = backup_id
         return defn.freeze(properties=props)
