@@ -16,20 +16,36 @@ from heat.common import grouputils
 from heat.common.i18n import _
 from heat.engine import attributes
 from heat.engine import constraints
+from heat.engine.hot import template
 from heat.engine import properties
 from heat.engine.resources.aws.autoscaling import autoscaling_group as aws_asg
+from heat.engine import rsrc_defn
 from heat.engine import support
-from heat.engine import template
+
+
+class HOTInterpreter(template.HOTemplate20150430):
+    def __new__(cls):
+        return object.__new__(cls)
+
+    def __init__(self):
+        version = {'heat_template_version': '2015-04-30'}
+        super(HOTInterpreter, self).__init__(version)
+
+    def parse(self, stack, snippet, path=''):
+        return snippet
+
+    def parse_conditions(self, stack, snippet, path=''):
+        return snippet
 
 
 class AutoScalingResourceGroup(aws_asg.AutoScalingGroup):
     """An autoscaling group that can scale arbitrary resources.
 
-    A resource allows to create desired count of similar resources, which
-    defined with resource property in HOT format. If there's necessity of
-    creating many of the same resources (e.g. one hundred sets of Server,
-    WaitCondition and WaitConditionHandle or even Neutron Nets),
-    AutoScalingGroup is the most convenient and easy way to do that.
+    An autoscaling group allows the creation of a desired count of similar
+    resources, which are defined with the resource property in HOT format.
+    If there is a need to create many of the same resources (e.g. one
+    hundred sets of Server, WaitCondition and WaitConditionHandle or even
+    Neutron Nets), AutoScalingGroup is a convenient and easy way to do that.
     """
 
     PROPERTIES = (
@@ -47,9 +63,9 @@ class AutoScalingResourceGroup(aws_asg.AutoScalingGroup):
     )
 
     ATTRIBUTES = (
-        OUTPUTS, OUTPUTS_LIST, CURRENT_SIZE,
+        OUTPUTS, OUTPUTS_LIST, CURRENT_SIZE, REFS, REFS_MAP,
     ) = (
-        'outputs', 'outputs_list', 'current_size',
+        'outputs', 'outputs_list', 'current_size', 'refs', 'refs_map',
     )
 
     properties_schema = {
@@ -121,14 +137,17 @@ class AutoScalingResourceGroup(aws_asg.AutoScalingGroup):
     attributes_schema = {
         OUTPUTS: attributes.Schema(
             _("A map of resource names to the specified attribute of each "
-              "individual resource. "
-              "Requires heat_template_version: 2014-10-16 or higher."),
+              "individual resource that is part of the AutoScalingGroup. "
+              "This map specifies output parameters that are available "
+              "once the AutoScalingGroup has been instantiated."),
             support_status=support.SupportStatus(version='2014.2'),
             type=attributes.Schema.MAP
         ),
         OUTPUTS_LIST: attributes.Schema(
-            _("A list of the specified attribute of each individual resource. "
-              "Requires heat_template_version: 2014-10-16 or higher."),
+            _("A list of the specified attribute of each individual resource "
+              "that is part of the AutoScalingGroup. This list of attributes "
+              "is available as an output once the AutoScalingGroup has been "
+              "instantiated."),
             support_status=support.SupportStatus(version='2014.2'),
             type=attributes.Schema.LIST
         ),
@@ -137,15 +156,26 @@ class AutoScalingResourceGroup(aws_asg.AutoScalingGroup):
             support_status=support.SupportStatus(version='2015.1'),
             type=attributes.Schema.INTEGER
         ),
+        REFS: attributes.Schema(
+            _("A list of resource IDs for the resources in the group."),
+            type=attributes.Schema.LIST,
+            support_status=support.SupportStatus(version='7.0.0'),
+        ),
+        REFS_MAP: attributes.Schema(
+            _("A map of resource names to IDs for the resources in "
+              "the group."),
+            type=attributes.Schema.MAP,
+            support_status=support.SupportStatus(version='7.0.0'),
+        ),
+
     }
     update_policy_schema = {}
 
-    def _get_resource_definition(self,
-                                 template_version=('heat_template_version',
-                                                   '2015-04-30')):
-        tmpl = template.Template(dict([template_version]))
-        return tmpl.rsrc_defn_from_snippet(None,
-                                           self.properties[self.RESOURCE])
+    def _get_resource_definition(self):
+        resource_def = self.properties[self.RESOURCE]
+        defn_data = dict(HOTInterpreter()._rsrc_defn_args(None, 'member',
+                                                          resource_def))
+        return rsrc_defn.ResourceDefinition(None, **defn_data)
 
     def _try_rolling_update(self, prop_diff):
         if self.RESOURCE in prop_diff:
@@ -165,6 +195,13 @@ class AutoScalingResourceGroup(aws_asg.AutoScalingGroup):
     def get_attribute(self, key, *path):
         if key == self.CURRENT_SIZE:
             return grouputils.get_size(self)
+        if key == self.REFS:
+            refs = grouputils.get_member_refids(self)
+            return refs
+        if key == self.REFS_MAP:
+            members = grouputils.get_members(self)
+            refs_map = {m.name: m.resource_id for m in members}
+            return refs_map
         if path:
             members = grouputils.get_members(self)
             attrs = ((rsrc.name, rsrc.FnGetAtt(*path)) for rsrc in members)
