@@ -16,6 +16,7 @@ import six
 
 from heat.common import exception
 from heat.common.i18n import _
+from heat.common.i18n import _LE
 from heat.common.i18n import _LI
 from heat.engine import attributes
 from heat.engine import constraints
@@ -142,14 +143,19 @@ class AutoScalingPolicy(signal_responder.SignalResponder,
                                                       self.context)
 
     def handle_signal(self, details=None):
-        # ceilometer sends details like this:
-        # {u'alarm_id': ID, u'previous': u'ok', u'current': u'alarm',
-        #  u'reason': u'...'})
-        # in this policy we currently assume that this gets called
-        # only when there is an alarm. But the template writer can
-        # put the policy in all the alarm notifiers (nodata, and ok).
+        # Template author can use scaling policy with any of the actions
+        # of an alarm (i.e alarm_actions, insufficient_data_actions) and
+        # it would be actioned irrespective of the alarm state. It's
+        # fair to assume that the alarm state would be the appropriate one.
+        # The responsibility of using a scaling policy with desired actions
+        # lies with the template author, though this is normally expected to
+        # be used with 'alarm_actions'.
         #
-        # our watchrule has upper case states so lower() them all.
+        # We also assume that the alarm state is 'alarm' when 'details' is None
+        # or no 'current'/'state' key in 'details'. Watchrule has upper case
+        # states, so we lower() them. This is only used for logging the alarm
+        # state.
+
         if details is None:
             alarm_state = 'alarm'
         else:
@@ -159,39 +165,43 @@ class AutoScalingPolicy(signal_responder.SignalResponder,
         LOG.info(_LI('Alarm %(name)s, new state %(state)s'),
                  {'name': self.name, 'state': alarm_state})
 
-        if alarm_state != 'alarm':
-            raise exception.NoActionRequired()
-        if not self._is_scaling_allowed():
-            LOG.info(_LI("%(name)s NOT performing scaling action, "
-                         "cooldown %(cooldown)s"),
-                     {'name': self.name,
-                      'cooldown': self.properties[self.COOLDOWN]})
-            raise exception.NoActionRequired()
-
         asgn_id = self.properties[self.AUTO_SCALING_GROUP_NAME]
         group = self.stack.resource_by_refid(asgn_id)
-        changed_size = False
-        try:
-            if group is None:
-                raise exception.NotFound(_('Alarm %(alarm)s could not find '
-                                           'scaling group named "%(group)s"'
-                                           ) % {'alarm': self.name,
-                                                'group': asgn_id})
 
-            LOG.info(_LI('%(name)s Alarm, adjusting Group %(group)s with id '
-                         '%(asgn_id)s by %(filter)s'),
-                     {'name': self.name, 'group': group.name,
-                      'asgn_id': asgn_id,
-                      'filter': self.properties[self.SCALING_ADJUSTMENT]})
-            changed_size = group.adjust(
+        if group is None:
+            raise exception.NotFound(_('Alarm %(alarm)s could not find '
+                                       'scaling group named "%(group)s"'
+                                       ) % {'alarm': self.name,
+                                            'group': asgn_id})
+
+        self._check_scaling_allowed()
+
+        LOG.info(_LI('%(name)s alarm, adjusting group %(group)s with id '
+                     '%(asgn_id)s by %(filter)s') % {
+            'name': self.name, 'group': group.name,
+            'asgn_id': asgn_id,
+            'filter': self.properties[self.SCALING_ADJUSTMENT]})
+
+        size_changed = False
+        try:
+            group.adjust(
                 self.properties[self.SCALING_ADJUSTMENT],
                 self.properties[self.ADJUSTMENT_TYPE],
                 self.properties[self.MIN_ADJUSTMENT_STEP])
+            size_changed = True
+        except resource.NoActionRequired:
+            raise
+        except Exception:
+            LOG.error(_LE("Error in performing scaling adjustment with "
+                          "%(name)s alarm for group %(group)s.") % {
+                'name': self.name,
+                'group': group.name})
+            raise
         finally:
             self._finished_scaling("%s : %s" % (
                 self.properties[self.ADJUSTMENT_TYPE],
                 self.properties[self.SCALING_ADJUSTMENT]),
-                changed_size=changed_size)
+                size_changed=size_changed)
 
     def _resolve_attribute(self, name):
         if self.resource_id is None:
