@@ -222,10 +222,12 @@ class Port(neutron.NeutronResource):
         ),
         MAC_ADDRESS: properties.Schema(
             properties.Schema.STRING,
-            _('MAC address to give to this port.'),
+            _('MAC address to give to this port. The default update policy '
+              'of this property in neutron is that allow admin role only.'),
             constraints=[
                 constraints.CustomConstraint('mac_addr')
-            ]
+            ],
+            update_allowed=True,
         ),
         ALLOWED_ADDRESS_PAIRS: properties.Schema(
             properties.Schema.LIST,
@@ -263,7 +265,8 @@ class Port(neutron.NeutronResource):
               'that this only works for Neutron deployments that support '
               'the bindings extension.'),
             constraints=[
-                constraints.AllowedValues(['normal', 'direct', 'macvtap']),
+                constraints.AllowedValues(['normal', 'direct', 'macvtap',
+                                           'direct-physical', 'baremetal']),
             ],
             support_status=support.SupportStatus(version='2015.1'),
             update_allowed=True
@@ -485,6 +488,8 @@ class Port(neutron.NeutronResource):
             return True
 
     def _resolve_attribute(self, name):
+        if self.resource_id is None:
+            return
         if name == self.SUBNETS_ATTR:
             subnets = []
             try:
@@ -533,8 +538,9 @@ class Port(neutron.NeutronResource):
     def restore_prev_rsrc(self, convergence=False):
         # In case of convergence, during rollback, the previous rsrc is
         # already selected and is being acted upon.
-        bakup_resources = self.stack._backup_stack().resources
-        prev_port = self if convergence else bakup_resources.get(self.name)
+        backup_stack = self.stack._backup_stack()
+        backup_res = backup_stack.resources.get(self.name)
+        prev_port = self if convergence else backup_res
         fixed_ips = prev_port.data().get('port_fip', [])
 
         props = {'fixed_ips': []}
