@@ -94,6 +94,11 @@ class StackResource(resource.Resource):
     def _needs_update(self, after, before, after_props, before_props,
                       prev_resource, check_init_complete=True):
 
+        # If stack resource is in CHECK_FAILED state, raise UpdateReplace
+        # to replace the failed stack.
+        if self.state == (self.CHECK, self.FAILED):
+            raise resource.UpdateReplace(self)
+
         # If the nested stack has not been created, use the default
         # implementation to determine if we need to replace the resource. Note
         # that we do *not* return the result.
@@ -102,21 +107,6 @@ class StackResource(resource.Resource):
                                                      after_props, before_props,
                                                      prev_resource,
                                                      check_init_complete)
-        else:
-            if self.state == (self.CHECK, self.FAILED):
-                nested_stack = self.rpc_client().show_stack(
-                    self.context, self.nested_identifier())[0]
-                nested_stack_state = (nested_stack[rpc_api.STACK_ACTION],
-                                      nested_stack[rpc_api.STACK_STATUS])
-                if nested_stack_state == (self.stack.CHECK, self.stack.FAILED):
-                    # The stack-check action marked the stack resource
-                    # CHECK_FAILED, so return True to allow the individual
-                    # CHECK_FAILED resources decide if they need updating.
-                    return True
-                # The mark-unhealthy action marked the stack resource
-                # CHECK_FAILED, so raise UpdateReplace to replace the
-                # entire failed stack.
-                raise resource.UpdateReplace(self)
 
         # Always issue an update to the nested stack and let the individual
         # resources in it decide if they need updating.
@@ -294,14 +284,14 @@ class StackResource(resource.Resource):
             'parent_resource_name': self.name
         })
         with self.translate_remote_exceptions:
-            result = None
             try:
                 result = self.rpc_client()._create_stack(self.context,
                                                          **kwargs)
-            finally:
-                if adopt_data is None and not result:
-                    raw_template.RawTemplate.delete(self.context,
-                                                    kwargs['template_id'])
+            except exception.HeatException:
+                with excutils.save_and_reraise_exception():
+                    if adopt_data is None:
+                        raw_template.RawTemplate.delete(self.context,
+                                                        kwargs['template_id'])
 
         self.resource_id_set(result['stack_id'])
 
@@ -495,12 +485,10 @@ class StackResource(resource.Resource):
             'args': {rpc_api.PARAM_TIMEOUT: timeout_mins}
         })
         with self.translate_remote_exceptions:
-            result = None
             try:
-                result = self.rpc_client()._update_stack(self.context,
-                                                         **kwargs)
-            finally:
-                if not result:
+                self.rpc_client()._update_stack(self.context, **kwargs)
+            except exception.HeatException:
+                with excutils.save_and_reraise_exception():
                     raw_template.RawTemplate.delete(self.context,
                                                     kwargs['template_id'])
         return cookie
@@ -597,10 +585,8 @@ class StackResource(resource.Resource):
         particular exception, not KeyError, being raised if the key does not
         exist.)
         """
-        if (self._outputs is None or
-                (op in self._outputs and
-                 rpc_api.OUTPUT_ERROR not in self._outputs[op] and
-                 self._outputs[op].get(rpc_api.OUTPUT_VALUE) is None)):
+        if self._outputs is None or self._outputs.get(op,
+                                                      NotImplemented) is None:
             stack_identity = self.nested_identifier()
             if stack_identity is None:
                 return
@@ -609,20 +595,14 @@ class StackResource(resource.Resource):
             if not stack:
                 return
             outputs = stack[0].get(rpc_api.STACK_OUTPUTS) or {}
-            self._outputs = {o[rpc_api.OUTPUT_KEY]: o for o in outputs}
+            self._outputs = {o[rpc_api.OUTPUT_KEY]: o[rpc_api.OUTPUT_VALUE]
+                             for o in outputs if rpc_api.OUTPUT_ERROR not in o}
 
-        if op not in self._outputs:
+        try:
+            return self._outputs[op]
+        except KeyError:
             raise exception.InvalidTemplateAttribute(resource=self.name,
                                                      key=op)
 
-        output_data = self._outputs[op]
-        if rpc_api.OUTPUT_ERROR in output_data:
-            raise exception.TemplateOutputError(
-                resource=self.name,
-                attribute=op,
-                message=output_data[rpc_api.OUTPUT_ERROR])
-
-        return output_data[rpc_api.OUTPUT_VALUE]
-
     def _resolve_attribute(self, name):
         return self.get_output(name)
