@@ -11,6 +11,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import functools
 import sys
 import types
 
@@ -19,11 +20,10 @@ from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
 import six
-from six import reraise as raise_
 
 from heat.common.i18n import _
 from heat.common.i18n import _LI
-from heat.common.i18n import repr_wraper
+from heat.common.i18n import repr_wrapper
 from heat.common import timeutils
 
 LOG = logging.getLogger(__name__)
@@ -48,6 +48,7 @@ def task_description(task):
     return encodeutils.safe_decode(repr(task))
 
 
+@functools.total_ordering
 class Timeout(BaseException):
     """Raised when task has exceeded its allotted (wallclock) running time.
 
@@ -79,28 +80,15 @@ class Timeout(BaseException):
             return False
 
     def __eq__(self, other):
-        return not self < other and not other < self
-
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
-    def __gt__(self, other):
-        return other < self
-
-    def __ge__(self, other):
-        return not self < other
-
-    def __le__(self, other):
-        return not other < self
+        if not isinstance(other, Timeout):
+            return NotImplemented
+        return not (self < other or other < self)
 
     def __lt__(self, other):
         if not isinstance(other, Timeout):
             return NotImplemented
         return self._duration.endtime() < other._duration.endtime()
 
-    def __cmp__(self, other):
-        return self < other
-
 
 class TimedCancel(Timeout):
     def trigger(self, generator):
@@ -140,6 +128,11 @@ class TaskRunner(object):
 
         The task function may be a co-routine that yields control flow between
         steps.
+
+        If the task co-routine wishes to be advanced only on every nth step of
+        the TaskRunner, it may yield an integer which is the period of the
+        task. e.g. "yield 2" will result in the task being advanced on every
+        second step.
         """
         assert callable(task), "Task is not callable"
 
@@ -149,6 +142,7 @@ class TaskRunner(object):
         self._runner = None
         self._done = False
         self._timeout = None
+        self._poll_period = 1
         self.name = task_description(task)
 
     def __str__(self):
@@ -162,18 +156,19 @@ class TaskRunner(object):
             LOG.debug('%s sleeping' % six.text_type(self))
             eventlet.sleep(wait_time)
 
-    def __call__(self, wait_time=1, timeout=None):
+    def __call__(self, wait_time=1, timeout=None, progress_callback=None):
         """Start and run the task to completion.
 
         The task will first sleep for zero seconds, then sleep for `wait_time`
         seconds between steps. To avoid sleeping, pass `None` for `wait_time`.
         """
-        self.start(timeout=timeout)
-        # ensure that zero second sleep is applied only if task
-        # has not completed.
-        if not self.done() and wait_time:
-            self._sleep(0)
-        self.run_to_completion(wait_time=wait_time)
+        assert self._runner is None, "Task already started"
+
+        started = False
+        for step in self.as_task(timeout=timeout,
+                                 progress_callback=progress_callback):
+            self._sleep(wait_time if (started or wait_time is None) else 0)
+            started = True
 
     def start(self, timeout=None):
         """Initialise the task and run its first step.
@@ -207,8 +202,12 @@ class TaskRunner(object):
         if not self.done():
             assert self._runner is not None, "Task not started"
 
+            if self._poll_period > 1:
+                self._poll_period -= 1
+                return False
+
             if self._timeout is not None and self._timeout.expired():
-                LOG.info(_LI('%s timed out'), six.text_type(self))
+                LOG.info(_LI('%s timed out'), self)
                 self._done = True
 
                 self._timeout.trigger(self._runner)
@@ -216,22 +215,61 @@ class TaskRunner(object):
                 LOG.debug('%s running' % six.text_type(self))
 
                 try:
-                    next(self._runner)
+                    poll_period = next(self._runner)
                 except StopIteration:
                     self._done = True
                     LOG.debug('%s complete' % six.text_type(self))
+                else:
+                    if isinstance(poll_period, six.integer_types):
+                        self._poll_period = max(poll_period, 1)
+                    else:
+                        self._poll_period = 1
 
         return self._done
 
-    def run_to_completion(self, wait_time=1):
+    def run_to_completion(self, wait_time=1, progress_callback=None):
         """Run the task to completion.
 
         The task will sleep for `wait_time` seconds between steps. To avoid
         sleeping, pass `None` for `wait_time`.
         """
-        while not self.step():
+        assert self._runner is not None, "Task not started"
+
+        for step in self.as_task(progress_callback=progress_callback):
             self._sleep(wait_time)
 
+    def as_task(self, timeout=None, progress_callback=None):
+        """Return a task that drives the TaskRunner."""
+        resuming = self.started()
+        if not resuming:
+            self.start(timeout=timeout)
+        else:
+            if timeout is not None:
+                new_timeout = Timeout(self, timeout)
+                if self._timeout is None or new_timeout < self._timeout:
+                    self._timeout = new_timeout
+
+        done = self.step() if resuming else self.done()
+        while not done:
+            try:
+                yield
+
+                if progress_callback is not None:
+                    progress_callback()
+            except GeneratorExit:
+                self.cancel()
+                raise
+            except:  # noqa
+                self._done = True
+                try:
+                    self._runner.throw(*sys.exc_info())
+                except StopIteration:
+                    return
+                else:
+                    self._done = False
+            else:
+                done = self.step()
+
     def cancel(self, grace_period=None):
         """Cancel the task and mark it as done."""
         if self.done():
@@ -291,7 +329,7 @@ def wrappertask(task):
 
         while True:
             try:
-                if subtask is not None:
+                if isinstance(subtask, types.GeneratorType):
                     subtask_running = True
                     try:
                         step = next(subtask)
@@ -315,7 +353,7 @@ def wrappertask(task):
                             except StopIteration:
                                 subtask_running = False
                 else:
-                    yield
+                    yield subtask
             except GeneratorExit:
                 parent.close()
                 raise
@@ -333,7 +371,7 @@ def wrappertask(task):
     return wrapper
 
 
-@repr_wraper
+@repr_wrapper
 class DependencyTaskGroup(object):
     """Task which manages group of subtasks that have ordering dependencies."""
 
@@ -348,9 +386,12 @@ class DependencyTaskGroup(object):
         dependency tree is passed as an argument.
 
         If an error_wait_time is specified, tasks that are already running at
-        the time of an error will continue to run for up to the specified
-        time before being cancelled. Once all remaining tasks are complete or
-        have been cancelled, the original exception is raised.
+        the time of an error will continue to run for up to the specified time
+        before being cancelled. Once all remaining tasks are complete or have
+        been cancelled, the original exception is raised. If error_wait_time is
+        a callable function it will be called for each task, passing the
+        dependency key as an argument, to determine the error_wait_time for
+        that particular task.
 
         If aggregate_exceptions is True, then execution of parallel operations
         will not be cancelled in the event of an error (operations downstream
@@ -377,42 +418,72 @@ class DependencyTaskGroup(object):
     def __call__(self):
         """Return a co-routine which runs the task group."""
         raised_exceptions = []
-        while any(six.itervalues(self._runners)):
-            try:
-                for k, r in self._ready():
-                    r.start()
-                    if not r:
-                        del self._graph[k]
+        thrown_exceptions = []
 
-                yield
+        try:
+            while any(six.itervalues(self._runners)):
+                try:
+                    for k, r in self._ready():
+                        r.start()
+                        if not r:
+                            del self._graph[k]
 
-                for k, r in self._running():
-                    if r.step():
-                        del self._graph[k]
-            except Exception:
-                exc_info = sys.exc_info()
+                    if self._graph:
+                        try:
+                            yield
+                        except Exception:
+                            thrown_exceptions.append(sys.exc_info())
+                            raise
+
+                    for k, r in self._running():
+                        if r.step():
+                            del self._graph[k]
+                except Exception:
+                    exc_info = None
+                    try:
+                        exc_info = sys.exc_info()
+                        if self.aggregate_exceptions:
+                            self._cancel_recursively(k, r)
+                        else:
+                            self.cancel_all(grace_period=self.error_wait_time)
+                        raised_exceptions.append(exc_info)
+                    finally:
+                        del exc_info
+                except:  # noqa
+                    with excutils.save_and_reraise_exception():
+                        self.cancel_all()
+
+            if raised_exceptions:
                 if self.aggregate_exceptions:
-                    self._cancel_recursively(k, r)
+                    raise ExceptionGroup(v for t, v, tb in raised_exceptions)
                 else:
-                    self.cancel_all(grace_period=self.error_wait_time)
-                raised_exceptions.append(exc_info)
-            except:  # noqa
-                with excutils.save_and_reraise_exception():
-                    self.cancel_all()
-
-        if raised_exceptions:
-            if self.aggregate_exceptions:
-                raise ExceptionGroup(v for t, v, tb in raised_exceptions)
-            else:
-                exc_type, exc_val, traceback = raised_exceptions[0]
-                raise_(exc_type, exc_val, traceback)
+                    if thrown_exceptions:
+                        six.reraise(*thrown_exceptions[-1])
+                    else:
+                        six.reraise(*raised_exceptions[0])
+        finally:
+            del raised_exceptions
+            del thrown_exceptions
 
     def cancel_all(self, grace_period=None):
-        for r in six.itervalues(self._runners):
-            r.cancel(grace_period=grace_period)
+        if callable(grace_period):
+            get_grace_period = grace_period
+        else:
+            def get_grace_period(key):
+                return grace_period
+
+        for k, r in six.iteritems(self._runners):
+            gp = get_grace_period(k)
+            try:
+                r.cancel(grace_period=gp)
+            except Exception as ex:
+                LOG.debug('Exception cancelling task: %s' % six.text_type(ex))
 
     def _cancel_recursively(self, key, runner):
-        runner.cancel()
+        try:
+            runner.cancel()
+        except Exception as ex:
+            LOG.debug('Exception cancelling task: %s' % six.text_type(ex))
         node = self._graph[key]
         for dependent_node in node.required_by():
             node_runner = self._runners[dependent_node]
