@@ -15,7 +15,6 @@ import uuid
 
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
-from oslo_service import service
 from oslo_utils import timeutils
 import requests
 import six
@@ -25,7 +24,7 @@ from heat.common import crypt
 from heat.common import exception
 from heat.common.i18n import _
 from heat.common.i18n import _LI
-from heat.db import api as db_api
+from heat.db.sqlalchemy import api as db_api
 from heat.engine import api
 from heat.engine import scheduler
 from heat.engine import software_config_io as swc_io
@@ -37,7 +36,7 @@ from heat.rpc import api as rpc_api
 LOG = logging.getLogger(__name__)
 
 
-class SoftwareConfigService(service.Service):
+class SoftwareConfigService(object):
 
     def show_software_config(self, cnxt, config_id):
         sc = software_config_object.SoftwareConfig.get_by_id(cnxt, config_id)
@@ -48,7 +47,9 @@ class SoftwareConfigService(service.Service):
             cnxt,
             limit=limit,
             marker=marker)
-        result = [api.format_software_config(sc, detail=False) for sc in scs]
+        result = [api.format_software_config(sc, detail=False,
+                                             include_project=cnxt.is_admin)
+                  for sc in scs]
         return result
 
     def create_software_config(self, cnxt, group, name, config,
@@ -101,7 +102,6 @@ class SoftwareConfigService(service.Service):
         rs = db_api.resource_get_by_physical_resource_id(cnxt, server_id)
         if not rs:
             return
-        cnxt.session.refresh(rs)
         deployments = self.metadata_software_deployments(cnxt, server_id)
         md = rs.rsrc_metadata or {}
         md['deployments'] = deployments
@@ -256,7 +256,7 @@ class SoftwareConfigService(service.Service):
             'stack_user_project_id': stack_user_project_id,
             'action': action,
             'status': status,
-            'status_reason': status_reason})
+            'status_reason': str(status_reason)})
         self._push_metadata_software_deployments(
             cnxt, server_id, stack_user_project_id)
         return api.format_software_deployment(sd)
@@ -337,7 +337,7 @@ class SoftwareConfigService(service.Service):
         if status:
             update_data['status'] = status
         if status_reason:
-            update_data['status_reason'] = status_reason
+            update_data['status_reason'] = str(status_reason)
         if updated_at:
             update_data['updated_at'] = timeutils.normalize_time(
                 timeutils.parse_isotime(updated_at))
