@@ -20,6 +20,7 @@ import itertools
 import re
 import warnings
 
+from keystoneauth1 import exceptions as ks_exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
@@ -221,11 +222,12 @@ class Stack(collections.Mapping):
 
         resources.initialise()
 
-        self.parameters = self.t.parameters(
-            self.identifier(),
-            user_params=self.env.params,
-            param_defaults=self.env.param_defaults)
-        self._set_param_stackid()
+        if self.t is not None:
+            self.parameters = self.t.parameters(
+                self.identifier(),
+                user_params=self.env.params,
+                param_defaults=self.env.param_defaults)
+            self._set_param_stackid()
 
     @property
     def tags(self):
@@ -313,6 +315,8 @@ class Stack(collections.Mapping):
     def _find_filtered_resources(self, filters=None):
         rsrc_def_cache = {self.t.id: self.t.resource_definitions(self)}
         if filters:
+            assert self.cache_data is None, \
+                "Resources should not be loaded from the DB"
             resources = resource_objects.Resource.get_all_by_stack(
                 self.context, self.id, filters)
         else:
@@ -352,6 +356,8 @@ class Stack(collections.Mapping):
 
     def _db_resources_get(self):
         if self._db_resources is None:
+            assert self.cache_data is None, \
+                "Resources should not be loaded from the DB"
             _db_resources = resource_objects.Resource.get_all_by_stack(
                 self.context, self.id)
             if not _db_resources:
@@ -364,19 +370,15 @@ class Stack(collections.Mapping):
         if tid is None:
             tid = self.t.id
 
-        if tid == self.t.id:
-            cur_res = self.resources.get(db_res.name)
-            if cur_res is not None and (cur_res.id == db_res.id):
-                return cur_res
-
         if rsrc_def_cache and tid in rsrc_def_cache:
             rsrc_def = rsrc_def_cache[tid]
+        elif tid == self.t.id:
+            rsrc_def = self.t.resource_definitions(self)
+            if rsrc_def_cache:
+                rsrc_def_cache[tid] = rsrc_def
         else:
-            if tid == self.t.id:
-                rsrc_def = self.t.resource_definitions(self)
-            else:
-                t = tmpl.Template.load(self.context, tid)
-                rsrc_def = t.resource_definitions(self)
+            t = tmpl.Template.load(self.context, tid)
+            rsrc_def = t.resource_definitions(self)
             if rsrc_def_cache:
                 rsrc_def_cache[tid] = rsrc_def
 
@@ -399,7 +401,6 @@ class Stack(collections.Mapping):
     def dependencies(self):
         if self._dependencies is None:
             self._dependencies = self._get_dependencies(
-                six.itervalues(self.resources),
                 ignore_errors=self.id is not None)
         return self._dependencies
 
@@ -470,13 +471,13 @@ class Stack(collections.Mapping):
         return set(itertools.chain.from_iterable(
             res.dep_attrs(resource_name) for res in resources))
 
-    @staticmethod
-    def _get_dependencies(resources, ignore_errors=True):
+    def _get_dependencies(self, ignore_errors=True):
         """Return the dependency graph for a list of resources."""
         deps = dependencies.Dependencies()
-        for res in resources:
+        for res in six.itervalues(self.resources):
             res.add_explicit_dependencies(deps)
 
+        for res in six.itervalues(self.resources):
             try:
                 res.add_dependencies(deps)
             except Exception as exc:
@@ -493,8 +494,8 @@ class Stack(collections.Mapping):
     @classmethod
     def load(cls, context, stack_id=None, stack=None, show_deleted=True,
              use_stored_context=False, force_reload=False, cache_data=None,
-             service_check_defer=False,
-             resource_validate=True):
+             service_check_defer=False, resource_validate=True,
+             load_template=True):
         """Retrieve a Stack from the database."""
         if stack is None:
             stack = stack_object.Stack.get_by_id(
@@ -512,7 +513,8 @@ class Stack(collections.Mapping):
                             use_stored_context=use_stored_context,
                             cache_data=cache_data,
                             service_check_defer=service_check_defer,
-                            resource_validate=resource_validate)
+                            resource_validate=resource_validate,
+                            load_template=load_template)
 
     @classmethod
     def load_all(cls, context, limit=None, marker=None, sort_keys=None,
@@ -533,7 +535,8 @@ class Stack(collections.Mapping):
             tags=tags,
             tags_any=tags_any,
             not_tags=not_tags,
-            not_tags_any=not_tags_any)
+            not_tags_any=not_tags_any,
+            eager_load=True)
         for stack in stacks:
             try:
                 yield cls._from_db(context, stack)
@@ -545,9 +548,13 @@ class Stack(collections.Mapping):
     @classmethod
     def _from_db(cls, context, stack,
                  use_stored_context=False, cache_data=None,
-                 service_check_defer=False, resource_validate=True):
-        template = tmpl.Template.load(
-            context, stack.raw_template_id, stack.raw_template)
+                 service_check_defer=False, resource_validate=True,
+                 load_template=True):
+        if load_template:
+            template = tmpl.Template.load(
+                context, stack.raw_template_id, stack.raw_template)
+        else:
+            template = None
         return cls(context, stack.name, template,
                    stack_id=stack.id,
                    action=stack.action, status=stack.status,
@@ -600,7 +607,7 @@ class Stack(collections.Mapping):
             stack.update({
                 'action': self.action,
                 'status': self.status,
-                'status_reason': self.status_reason})
+                'status_reason': str(self.status_reason)})
 
         if only_db:
             stack['parent_resource_name'] = self.parent_resource_name
@@ -749,7 +756,8 @@ class Stack(collections.Mapping):
                     (r.RESUME, r.IN_PROGRESS),
                     (r.RESUME, r.COMPLETE),
                     (r.UPDATE, r.IN_PROGRESS),
-                    (r.UPDATE, r.COMPLETE)) and
+                    (r.UPDATE, r.COMPLETE),
+                    (r.CHECK, r.COMPLETE)) and
                     (r.FnGetRefId() == refid or r.name == refid)):
                 return r
 
@@ -774,7 +782,8 @@ class Stack(collections.Mapping):
         return handler and handler(resource_name)
 
     @profiler.trace('Stack.validate', hide_args=False)
-    def validate(self, ignorable_errors=None, validate_by_deps=True):
+    def validate(self, ignorable_errors=None, validate_by_deps=True,
+                 validate_resources=True):
         """Validates the stack."""
         # TODO(sdake) Should return line number of invalid reference
 
@@ -821,38 +830,49 @@ class Stack(collections.Mapping):
         else:
             iter_rsc = six.itervalues(resources)
 
-        for res in iter_rsc:
-            try:
-                if self.resource_validate:
-                    result = res.validate()
-                else:
-                    result = res.validate_template()
-            except exception.HeatException as ex:
-                LOG.debug('%s', ex)
-                if ignorable_errors and ex.error_code in ignorable_errors:
-                    result = None
-                else:
+        # Validate resources only for top-level stacks. Nested stacks have
+        # already had their resources validated by their parent.
+        if validate_resources:
+            unique_defns = set(res.t for res in six.itervalues(resources))
+            unique_defn_names = set(defn.name for defn in unique_defns)
+
+            for res in iter_rsc:
+                # Don't validate identical definitions multiple times
+                if res.name not in unique_defn_names:
+                    continue
+                try:
+                    if self.resource_validate:
+                        if res.external_id is not None:
+                            res.validate_external()
+                            continue
+                        result = res.validate()
+                    elif res.external_id is None:
+                        result = res.validate_template()
+                except exception.HeatException as ex:
+                    LOG.debug('%s', ex)
+                    if ignorable_errors and ex.error_code in ignorable_errors:
+                        result = None
+                    else:
+                        raise
+                except AssertionError:
                     raise
-            except AssertionError:
-                raise
-            except Exception as ex:
-                LOG.info(_LI("Exception in stack validation"), exc_info=True)
-                raise exception.StackValidationFailed(
-                    message=encodeutils.safe_decode(six.text_type(ex)))
-            if result:
-                raise exception.StackValidationFailed(message=result)
+                except Exception as ex:
+                    LOG.info(_LI("Exception in stack validation"),
+                             exc_info=True)
+                    raise exception.StackValidationFailed(
+                        message=encodeutils.safe_decode(six.text_type(ex)))
+                if result:
+                    raise exception.StackValidationFailed(message=result)
 
         for op_name, output in six.iteritems(self.outputs):
             try:
-                output.validate()
-            except exception.StackValidationFailed as ex:
+                path = '.'.join([self.t.OUTPUTS, op_name,
+                                 self.t.OUTPUT_VALUE])
+                output.validate(path)
+            except exception.StackValidationFailed:
                 raise
             except AssertionError:
                 raise
-            except Exception as ex:
-                raise exception.StackValidationFailed(
-                    error='Validation error in output "%s"' % op_name,
-                    message=six.text_type(ex))
 
     def requires_deferred_auth(self):
         """Determine whether to perform API requests with deferred auth.
@@ -866,7 +886,7 @@ class Stack(collections.Mapping):
     def _add_event(self, action, status, reason):
         """Add a state change event to the database."""
         ev = event.Event(self.context, self, action, status, reason,
-                         self.id, {},
+                         self.id, None,
                          self.name, 'OS::Heat::Stack')
 
         ev.store()
@@ -925,11 +945,12 @@ class Stack(collections.Mapping):
         """Persist stack state to database"""
         if self.id is None:
             return
-        stack = stack_object.Stack.get_by_id(self.context, self.id)
+        stack = stack_object.Stack.get_by_id(self.context, self.id,
+                                             eager_load=False)
         if stack is not None:
             values = {'action': self.action,
                       'status': self.status,
-                      'status_reason': self.status_reason}
+                      'status_reason': str(self.status_reason)}
             self._send_notification_and_add_event()
             if self.convergence:
                 # do things differently for convergence
@@ -956,11 +977,12 @@ class Stack(collections.Mapping):
         """Persist stack state to database and release stack lock"""
         if self.id is None:
             return
-        stack = stack_object.Stack.get_by_id(self.context, self.id)
+        stack = stack_object.Stack.get_by_id(self.context, self.id,
+                                             eager_load=False)
         if stack is not None:
             values = {'action': self.action,
                       'status': self.status,
-                      'status_reason': self.status_reason}
+                      'status_reason': str(self.status_reason)}
             self._send_notification_and_add_event()
             stack.persist_state_and_release_lock(self.context, self.id,
                                                  engine_id, values)
@@ -1405,6 +1427,14 @@ class Stack(collections.Mapping):
 
         return self._convg_deps
 
+    def reset_stack_and_resources_in_progress(self, reason):
+        for name, rsrc in six.iteritems(self.resources):
+            if rsrc.status == rsrc.IN_PROGRESS:
+                rsrc.state_set(rsrc.action,
+                               rsrc.FAILED,
+                               six.text_type(reason))
+        self.state_set(self.action, self.FAILED, six.text_type(reason))
+
     @scheduler.wrappertask
     def update_task(self, newstack, action=UPDATE, msg_queue=None):
         if action not in (self.UPDATE, self.ROLLBACK, self.RESTORE):
@@ -1424,8 +1454,9 @@ class Stack(collections.Mapping):
             if action == self.ROLLBACK:
                 LOG.debug("Starting update rollback for %s" % self.name)
             else:
-                self.state_set(action, self.FAILED,
-                               'State invalid for %s' % action)
+                reason = _('Attempted to %s an IN_PROGRESS '
+                           'stack') % action
+                self.reset_stack_and_resources_in_progress(reason)
                 return
 
         # Save a copy of the new template.  To avoid two DB writes
@@ -1523,14 +1554,9 @@ class Stack(collections.Mapping):
                 # and new stack resources, we should have user params of both.
                 existing_params.load(newstack.t.env.user_env_as_dict())
                 self.t.env = existing_params
-                # Update the template version, in case new things were used
-                self.t.t[newstack.t.version[0]] = max(
-                    newstack.t.version[1], self.t.version[1])
                 self.t.merge_snippets(newstack.t)
                 self.t.store(self.context)
                 backup_stack.t.env = existing_params
-                backup_stack.t.t[newstack.t.version[0]] = max(
-                    newstack.t.version[1], self.t.version[1])
                 backup_stack.t.merge_snippets(newstack.t)
                 backup_stack.t.store(self.context)
             self.store()
@@ -1668,6 +1694,11 @@ class Stack(collections.Mapping):
                         else:
                             self.clients.client('keystone').delete_trust(
                                 trust_id)
+                    except ks_exceptions.Unauthorized:
+                        # ignore if unauthorized, don't to set FAILED
+                        LOG.warning(_LW("Unauthorized, including the cases: "
+                                        "the trustor is deleted or the trust "
+                                        "is expired."))
                     except Exception as ex:
                         LOG.exception(_LE("Error deleting trust"))
                         stack_status = self.FAILED
@@ -1807,7 +1838,7 @@ class Stack(collections.Mapping):
         """
         # No need to suspend if the stack has been suspended
         if self.state == (self.SUSPEND, self.COMPLETE):
-            LOG.info(_LI('%s is already suspended'), six.text_type(self))
+            LOG.info(_LI('%s is already suspended'), self)
             return
 
         self.updated_time = oslo_timeutils.utcnow()
@@ -1832,7 +1863,7 @@ class Stack(collections.Mapping):
         """
         # No need to resume if the stack has been resumed
         if self.state == (self.RESUME, self.COMPLETE):
-            LOG.info(_LI('%s is already resumed'), six.text_type(self))
+            LOG.info(_LI('%s is already resumed'), self)
             return
 
         self.updated_time = oslo_timeutils.utcnow()
@@ -1967,13 +1998,6 @@ class Stack(collections.Mapping):
             'tags': self.tags,
         }
 
-    def resolve_static_data(self, snippet, path=''):
-        warnings.warn('Stack.resolve_static_data() is deprecated and '
-                      'will be removed in the Ocata release.',
-                      DeprecationWarning)
-
-        return self.t.parse(self, snippet, path=path)
-
     def reset_resource_attributes(self):
         # nothing is cached if no resources exist
         if not self._resources:
@@ -1996,8 +2020,7 @@ class Stack(collections.Mapping):
             resource_name, {}).get('attrs', {}).get(attribute_key)
 
     def cache_data_resource_all_attributes(self, resource_name):
-        attrs = self.cache_data.get(resource_name, {}).get('attributes', {})
-        return attrs
+        return self.cache_data.get(resource_name, {}).get('attrs', {})
 
     def mark_complete(self):
         """Mark the update as complete.
