@@ -17,7 +17,6 @@ import eventlet.queue
 
 from oslo_log import log as logging
 import oslo_messaging
-from oslo_service import service
 from oslo_utils import uuidutils
 from osprofiler import profiler
 
@@ -26,7 +25,7 @@ from heat.common.i18n import _LE
 from heat.common.i18n import _LI
 from heat.common.i18n import _LW
 from heat.common import messaging as rpc_messaging
-from heat.db import api as db_api
+from heat.db.sqlalchemy import api as db_api
 from heat.engine import check_resource
 from heat.engine import stack as parser
 from heat.engine import sync_point
@@ -40,7 +39,7 @@ CANCEL_RETRIES = 3
 
 
 @profiler.trace_cls("rpc")
-class WorkerService(service.Service):
+class WorkerService(object):
     """Service that has 'worker' actor in convergence.
 
     This service is dedicated to handle internal messages to the 'worker'
@@ -57,7 +56,6 @@ class WorkerService(service.Service):
                  topic,
                  engine_id,
                  thread_group_mgr):
-        super(WorkerService, self).__init__()
         self.host = host
         self.topic = topic
         self.engine_id = engine_id
@@ -81,8 +79,6 @@ class WorkerService(service.Service):
         self._rpc_server = rpc_messaging.get_rpc_server(target, self)
         self._rpc_server.start()
 
-        super(WorkerService, self).start()
-
     def stop(self):
         if self._rpc_server is None:
             return
@@ -96,8 +92,6 @@ class WorkerService(service.Service):
             LOG.error(_LE("%(topic)s is failed to stop, %(exc)s"),
                       {'topic': self.topic, 'exc': e})
 
-        super(WorkerService, self).stop()
-
     def stop_traversal(self, stack):
         """Update current traversal to stop workers from propagating.
 
@@ -114,7 +108,8 @@ class WorkerService(service.Service):
             if db_child.status == parser.Stack.IN_PROGRESS:
                 child = parser.Stack.load(stack.context,
                                           stack_id=db_child.id,
-                                          stack=db_child)
+                                          stack=db_child,
+                                          load_template=False)
                 _stop_traversal(child)
 
     def stop_all_workers(self, stack):
@@ -209,8 +204,8 @@ def _stop_traversal(stack):
     reason = 'Stack %(action)s cancelled' % {'action': stack.action}
     updated = stack.state_set(stack.action, stack.FAILED, reason)
     if not updated:
-        LOG.warning(_LW("Failed to update stack %(name)s status"
-                        " to %(action)_%(state)"),
+        LOG.warning(_LW("Failed to update stack %(name)s status "
+                        "to %(action)s_%(state)s"),
                     {'name': stack.name, 'action': stack.action,
                     'state': stack.FAILED})
         return
