@@ -25,7 +25,7 @@ from oslo_versionedobjects import fields
 from heat.common import crypt
 from heat.common import environment_format as env_fmt
 from heat.common.i18n import _LW
-from heat.db import api as db_api
+from heat.db.sqlalchemy import api as db_api
 from heat.objects import base as heat_base
 from heat.objects import fields as heat_fields
 
