@@ -18,30 +18,34 @@
 import collections
 
 from oslo_config import cfg
-from oslo_serialization import jsonutils
+from oslo_log import log as logging
 from oslo_versionedobjects import base
 from oslo_versionedobjects import fields
-import retrying
 import six
+import tenacity
 
 from heat.common import crypt
 from heat.common import exception
 from heat.common.i18n import _
-from heat.db import api as db_api
+from heat.common.i18n import _LE
+from heat.db.sqlalchemy import api as db_api
 from heat.objects import base as heat_base
 from heat.objects import fields as heat_fields
 from heat.objects import resource_data
+from heat.objects import resource_properties_data as rpd
 
 cfg.CONF.import_opt('encrypt_parameters_and_properties', 'heat.common.config')
 
+LOG = logging.getLogger(__name__)
+
 
 def retry_on_conflict(func):
-    def is_conflict(ex):
-        return isinstance(ex, exception.ConcurrentTransaction)
-
-    wrapper = retrying.retry(stop_max_attempt_number=11,
-                             wait_random_min=0, wait_random_max=2000,
-                             retry_on_exception=is_conflict)
+    wrapper = tenacity.retry(
+        stop=tenacity.stop_after_attempt(11),
+        wait=tenacity.wait_random(max=0.002),
+        retry=tenacity.retry_if_exception_type(
+            exception.ConcurrentTransaction),
+        reraise=True)
     return wrapper(func)
 
 
@@ -75,12 +79,14 @@ class Resource(
         'status_reason': fields.StringField(nullable=True),
         'action': fields.StringField(nullable=True),
         'rsrc_metadata': heat_fields.JsonField(nullable=True),
-        'properties_data': heat_fields.JsonField(nullable=True),
-        'properties_data_encrypted': fields.BooleanField(default=False),
         'data': fields.ListOfObjectsField(
             resource_data.ResourceData,
             nullable=True
         ),
+        'rsrc_prop_data': fields.ObjectField(
+            rpd.ResourcePropertiesData, nullable=True),
+        'rsrc_prop_data_id': fields.ObjectField(
+            fields.IntegerField(nullable=True)),
         'engine_id': fields.StringField(nullable=True),
         'atomic_key': fields.IntegerField(nullable=True),
         'current_template_id': fields.IntegerField(),
@@ -103,19 +109,38 @@ class Resource(
             else:
                 resource[field] = db_resource[field]
 
-        if resource.properties_data_encrypted and resource.properties_data:
-            properties_data = {}
-            for prop_name, prop_value in resource.properties_data.items():
-                method, value = prop_value
-                decrypted_value = crypt.decrypt(method, value)
-                prop_string = jsonutils.loads(decrypted_value)
-                properties_data[prop_name] = prop_string
-            resource.properties_data = properties_data
+        if db_resource['rsrc_prop_data'] is not None:
+            resource['rsrc_prop_data'] = \
+                rpd.ResourcePropertiesData._from_db_object(
+                    rpd.ResourcePropertiesData(context), context,
+                    db_resource['rsrc_prop_data'])
+            resource._properties_data = resource['rsrc_prop_data'].data
+            if db_resource['properties_data']:
+                LOG.error(
+                    _LE('Unexpected condition where resource.rsrc_prop_data '
+                        'and resource.properties_data are both not null. '
+                        'rsrc_prop_data.id: %(rsrc_prop_data_id)s ,'
+                        'resource id: %(res_id)s')
+                    % {'rsrc_prop_data_id': resource['rsrc_prop_data'].id,
+                       'res_id': resource['id']})
+        elif db_resource['properties_data']:  # legacy field
+            if db_resource['properties_data_encrypted']:
+                decrypted_data = crypt.decrypted_dict(
+                    db_resource['properties_data'])
+                resource._properties_data = decrypted_data
+            else:
+                resource._properties_data = db_resource['properties_data']
+        else:
+            resource._properties_data = {}
 
         resource._context = context
         resource.obj_reset_changes()
         return resource
 
+    @property
+    def properties_data(self):
+        return self._properties_data
+
     @classmethod
     def get_obj(cls, context, resource_id, refresh=False):
         resource_db = db_api.resource_get(context, resource_id,
@@ -208,11 +233,12 @@ class Resource(
         return cls._from_db_object(cls(context), context, resource_db)
 
     @classmethod
-    def get_by_physical_resource_id(cls, context, physical_resource_id):
-        resource_db = db_api.resource_get_by_physical_resource_id(
+    def get_all_by_physical_resource_id(cls, context, physical_resource_id):
+        matches = db_api.resource_get_all_by_physical_resource_id(
             context,
             physical_resource_id)
-        return cls._from_db_object(cls(context), context, resource_db)
+        return [cls._from_db_object(cls(context), context, resource_db)
+                for resource_db in matches]
 
     @classmethod
     def update_by_id(cls, context, resource_id, values):
@@ -237,11 +263,7 @@ class Resource(
     @staticmethod
     def encrypt_properties_data(data):
         if cfg.CONF.encrypt_parameters_and_properties and data:
-            result = {}
-            for prop_name, prop_value in data.items():
-                prop_string = jsonutils.dumps(prop_value)
-                encrypted_value = crypt.encrypt(prop_string)
-                result[prop_name] = encrypted_value
+            result = crypt.encrypted_dict(data)
             return (True, result)
         return (False, data)
 
