@@ -13,7 +13,7 @@
 
 import six
 
-from heat.db import api as db_api
+from heat.db.sqlalchemy import api as db_api
 from heat.engine import service
 from heat.engine import stack
 from heat.tests.convergence.framework import message_processor
@@ -22,16 +22,28 @@ from heat.tests.convergence.framework import scenario_template
 from heat.tests import utils
 
 
+class SynchronousThreadGroupManager(service.ThreadGroupManager):
+    """Wrapper for thread group manager.
+
+    The start method of thread group manager needs to be overridden to
+    run the function synchronously so the convergence scenario
+    tests can be run.
+    """
+    def start(self, stack_id, func, *args, **kwargs):
+        func(*args, **kwargs)
+
+
 class Engine(message_processor.MessageProcessor):
-    """Wrapper to the engine service.
+    """Wrapper for the engine service.
 
     Methods of this class will be called from the scenario tests.
     """
 
     queue = message_queue.MessageQueue('engine')
 
-    def __init__(self):
+    def __init__(self, worker):
         super(Engine, self).__init__('engine')
+        self.worker = worker
 
     def scenario_template_to_hot(self, scenario_tmpl):
         """Converts the scenario template into hot template."""
@@ -63,8 +75,8 @@ class Engine(message_processor.MessageProcessor):
     def create_stack(self, stack_name, scenario_tmpl):
         cnxt = utils.dummy_context()
         srv = service.EngineService("host", "engine")
-        thread_group_mgr = service.ThreadGroupManager()
-        srv.thread_group_mgr = thread_group_mgr
+        srv.thread_group_mgr = SynchronousThreadGroupManager()
+        srv.worker_service = self.worker
         hot_tmpl = self.scenario_template_to_hot(scenario_tmpl)
         srv.create_stack(cnxt, stack_name, hot_tmpl,
                          params={}, files={}, environment_files=None, args={})
@@ -74,8 +86,8 @@ class Engine(message_processor.MessageProcessor):
         cnxt = utils.dummy_context()
         db_stack = db_api.stack_get_by_name(cnxt, stack_name)
         srv = service.EngineService("host", "engine")
-        thread_group_mgr = service.ThreadGroupManager()
-        srv.thread_group_mgr = thread_group_mgr
+        srv.thread_group_mgr = SynchronousThreadGroupManager()
+        srv.worker_service = self.worker
         hot_tmpl = self.scenario_template_to_hot(scenario_tmpl)
         stack_identity = {'stack_name': stack_name,
                           'stack_id': db_stack.id,
@@ -93,6 +105,8 @@ class Engine(message_processor.MessageProcessor):
                           'tenant': db_stack.tenant,
                           'path': ''}
         srv = service.EngineService("host", "engine")
+        srv.thread_group_mgr = SynchronousThreadGroupManager()
+        srv.worker_service = self.worker
         srv.delete_stack(cnxt, stack_identity)
 
     @message_processor.asynchronous
