@@ -23,7 +23,7 @@ import six
 from heat.common import crypt
 from heat.common import exception
 from heat.common import template_format
-from heat.db import api as db_api
+from heat.db.sqlalchemy import api as db_api
 from heat.engine.clients.os import swift
 from heat.engine.clients.os import zaqar
 from heat.engine import service
@@ -81,12 +81,13 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
 
     def _create_software_config(
             self, group='Heat::Shell', name='config_mysql', config=None,
-            inputs=None, outputs=None, options=None):
+            inputs=None, outputs=None, options=None, context=None):
+        cntx = context if context else self.ctx
         inputs = inputs or []
         outputs = outputs or []
         options = options or {}
         return self.engine.create_software_config(
-            self.ctx, group, name, config, inputs, outputs, options)
+            cntx, group, name, config, inputs, outputs, options)
 
     def _create_dummy_config_object(self):
         obj_config = software_config_object.SoftwareConfig()
@@ -107,14 +108,27 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
 
     def test_list_software_configs(self):
         config = self._create_software_config()
-        config_id = config['id']
         self.assertIsNotNone(config)
+        config_id = config['id']
 
         configs = self.engine.list_software_configs(self.ctx)
         self.assertIsNotNone(configs)
         config_ids = [x['id'] for x in configs]
         self.assertIn(config_id, config_ids)
 
+        admin_cntx = utils.dummy_context(is_admin=True)
+
+        admin_config = self._create_software_config(context=admin_cntx)
+        admin_config_id = admin_config['id']
+        configs = self.engine.list_software_configs(admin_cntx)
+        self.assertIsNotNone(configs)
+        config_ids = [x['id'] for x in configs]
+        project_ids = [x['project'] for x in configs]
+        self.assertEqual(2, len(project_ids))
+        self.assertEqual(2, len(config_ids))
+        self.assertIn(config_id, config_ids)
+        self.assertIn(admin_config_id, config_ids)
+
     def test_show_software_config(self):
         config_id = str(uuid.uuid4())
 
@@ -223,10 +237,10 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
             self.ctx, server_id=server.resource_id)
         self.assertEqual([deployment], deployments)
 
-        rs = resource_objects.Resource.get_by_physical_resource_id(
+        rsrcs = resource_objects.Resource.get_all_by_physical_resource_id(
             self.ctx, server_id)
         self.assertEqual(deployment['config_id'],
-                         rs.rsrc_metadata.get('deployments')[0]['id'])
+                         rsrcs[0].rsrc_metadata.get('deployments')[0]['id'])
 
     def test_metadata_software_deployments(self):
         stack_name = 'test_metadata_software_deployments'
@@ -271,9 +285,9 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
 
         # assert that metadata via metadata_software_deployments matches
         # metadata via server resource
-        rs = resource_objects.Resource.get_by_physical_resource_id(
+        rsrcs = resource_objects.Resource.get_all_by_physical_resource_id(
             self.ctx, server_id)
-        self.assertEqual(metadata, rs.rsrc_metadata.get('deployments'))
+        self.assertEqual(metadata, rsrcs[0].rsrc_metadata.get('deployments'))
 
         deployments = self.engine.metadata_software_deployments(
             self.ctx, server_id=str(uuid.uuid4()))
@@ -635,9 +649,8 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
             'deployments': {'deploy': 'this'}
         }
 
-        with mock.patch.object(self.ctx.session, 'refresh'):
-            self.engine.software_config._push_metadata_software_deployments(
-                self.ctx, '1234', None)
+        self.engine.software_config._push_metadata_software_deployments(
+            self.ctx, '1234', None)
         res_upd.assert_called_once_with(
             self.ctx, '1234', {'rsrc_metadata': result_metadata}, 1)
         put.side_effect = Exception('Unexpected requests.put')
@@ -661,14 +674,12 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         deployments = {'deploy': 'this'}
         md_sd.return_value = deployments
 
-        with mock.patch.object(self.ctx.session, 'refresh'):
-            f = self.engine.software_config._push_metadata_software_deployments
-            self.assertRaises(
-                exception.ConcurrentTransaction,
-                f,
-                self.ctx,
-                '1234',
-                None)
+        self.assertRaises(
+            exception.ConcurrentTransaction,
+            self.engine.software_config._push_metadata_software_deployments,
+            self.ctx,
+            '1234',
+            None)
         # retry ten times then the final failure
         self.assertEqual(11, res_upd.call_count)
         put.assert_not_called()
@@ -698,9 +709,9 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
             'original': 'metadata',
             'deployments': {'deploy': 'this'}
         }
-        with mock.patch.object(self.ctx.session, 'refresh'):
-            self.engine.software_config._push_metadata_software_deployments(
-                self.ctx, '1234', None)
+
+        self.engine.software_config._push_metadata_software_deployments(
+            self.ctx, '1234', None)
         res_upd.assert_called_once_with(
             self.ctx, '1234', {'rsrc_metadata': result_metadata}, 1)
 
@@ -737,9 +748,8 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
             'deployments': {'deploy': 'this'}
         }
 
-        with mock.patch.object(self.ctx.session, 'refresh'):
-            self.engine.software_config._push_metadata_software_deployments(
-                self.ctx, '1234', 'project1')
+        self.engine.software_config._push_metadata_software_deployments(
+            self.ctx, '1234', 'project1')
         res_upd.assert_called_once_with(
             self.ctx, '1234', {'rsrc_metadata': result_metadata}, 1)
 
@@ -907,9 +917,8 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         zaqar_client.queue.return_value = queue
         queue.pop.return_value = [mock.Mock(body='ok')]
 
-        with mock.patch.object(self.ctx.session, 'refresh'):
-            deployment = self._create_software_deployment(
-                status='IN_PROGRESS', config_id=config['id'])
+        deployment = self._create_software_deployment(
+            status='IN_PROGRESS', config_id=config['id'])
 
         deployment_id = deployment['id']
         self.assertEqual(
