@@ -124,7 +124,7 @@ resources:
 '''
 
 attr_cache_template = '''
-heat_template_version: 2013-05-23
+heat_template_version: 2016-04-08
 resources:
     A:
         type: ResourceWithComplexAttributesType
@@ -151,7 +151,7 @@ resources:
 
 
 def get_stack(stack_name, ctx, template=None, with_params=True,
-              convergence=False):
+              convergence=False, **kwargs):
     if template is None:
         t = template_format.parse(wp_template)
         if with_params:
@@ -162,7 +162,8 @@ def get_stack(stack_name, ctx, template=None, with_params=True,
     else:
         t = template_format.parse(template)
         tmpl = templatem.Template(t)
-    stack = parser.Stack(ctx, stack_name, tmpl, convergence=convergence)
+    stack = parser.Stack(ctx, stack_name, tmpl, convergence=convergence,
+                         **kwargs)
     return stack
 
 
@@ -192,7 +193,7 @@ def setup_mocks(mocks, stack, mock_image_constraint=True,
     metadata = instance.metadata_get()
     if mock_image_constraint:
         setup_mock_for_image_constraint(mocks,
-                                        instance.t['Properties']['ImageId'])
+                                        instance.properties['ImageId'])
 
     if mock_keystone:
         setup_keystone_mocks(mocks, stack)
@@ -203,7 +204,7 @@ def setup_mocks(mocks, stack, mock_image_constraint=True,
     mocks.StubOutWithMock(nova.NovaClientPlugin, 'build_userdata')
     nova.NovaClientPlugin.build_userdata(
         metadata,
-        instance.t['Properties']['UserData'],
+        user_data,
         'ec2-user').AndReturn(server_userdata)
 
     mocks.StubOutWithMock(fc.servers, 'create')
@@ -305,13 +306,9 @@ class DummyThreadGroup(object):
     def stop_timers(self):
         pass
 
-    def add_thread(self, callback, *args, **kwargs):
-        # just to make _start_with_trace() easier to test:
-        # callback == _start_with_trace
-        # args[0] == trace_info
-        # args[1] == actual_callback
-        callback = args[1]
-        self.threads.append(callback)
+    def add_thread(self, callback, cnxt, trace, func, *args, **kwargs):
+        # callback here is _start_with_trace(); func is the 'real' callback
+        self.threads.append(func)
         return DummyThread()
 
     def stop(self, graceful=False):
@@ -323,7 +320,7 @@ class DummyThreadGroup(object):
 
 class DummyThreadGroupManager(object):
     def __init__(self):
-        self.events = []
+        self.msg_queues = []
         self.messages = []
 
     def start_with_lock(self, cnxt, stack, engine_id, func, *args, **kwargs):
@@ -339,13 +336,13 @@ class DummyThreadGroupManager(object):
     def send(self, stack_id, message):
         self.messages.append(message)
 
-    def add_event(self, stack_id, event):
-        self.events.append(event)
+    def add_msg_queue(self, stack_id, msg_queue):
+        self.msg_queues.append(msg_queue)
 
-    def remove_event(self, gt, stack_id, event):
-        for e in self.events.pop(stack_id, []):
-            if e is not event:
-                self.add_event(stack_id, e)
+    def remove_msg_queue(self, gt, stack_id, msg_queue):
+        for q in self.msg_queues.pop(stack_id, []):
+            if q is not msg_queue:
+                self.add_event(stack_id, q)
 
 
 class DummyThreadGroupMgrLogStart(DummyThreadGroupManager):
