@@ -14,8 +14,8 @@
 
 import copy
 
-from heat.common import exception
 from heat.common import template_format
+from heat.engine import resource
 from heat.engine.resources.openstack.heat import cloud_watch
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
@@ -51,8 +51,6 @@ alarm_template = '''
 
 
 class CloudWatchAlarmTest(common.HeatTestCase):
-    def setUp(self):
-        super(CloudWatchAlarmTest, self).setUp()
 
     def create_alarm(self, t, stack, resource_name):
         resource_defns = stack.t.resource_definitions(stack)
@@ -127,7 +125,7 @@ class CloudWatchAlarmTest(common.HeatTestCase):
                                                props)
 
         updater = scheduler.TaskRunner(rsrc.update, snippet)
-        self.assertRaises(exception.UpdateReplace, updater)
+        self.assertRaises(resource.UpdateReplace, updater)
 
         scheduler.TaskRunner(rsrc.delete)()
         self.m.VerifyAll()
