@@ -78,10 +78,6 @@ Resources:
           min: 5
 '''
 
-    def setUp(self):
-        super(TestRandomString, self).setUp()
-        self.ctx = utils.dummy_context()
-
     def create_stack(self, templ):
         self.stack = self.parse_stack(template_format.parse(templ))
         self.assertIsNone(self.stack.create())
@@ -96,7 +92,7 @@ Resources:
         return stack
 
     def assert_min(self, pattern, string, minimum):
-        self.assertTrue(len(re.findall(pattern, string)) >= minimum)
+        self.assertGreaterEqual(len(re.findall(pattern, string)), minimum)
 
     def test_random_string(self):
         stack = self.create_stack(self.template_random_string)
