@@ -12,6 +12,7 @@
 #    under the License.
 
 import copy
+from magnumclient import exceptions as mc_exc
 import mock
 from oslo_config import cfg
 import six
@@ -26,12 +27,6 @@ from heat.engine import template
 from heat.tests import common
 from heat.tests import utils
 
-try:
-    from magnumclient.common.apiclient import exceptions as mc_exc
-except ImportError:
-    # NOTE (duvarenkov): For magnumclient versions before 2.0.0.
-    from magnumclient.openstack.common.apiclient import exceptions as mc_exc
-
 
 magnum_template = '''
     heat_template_version: 2015-04-30
@@ -53,7 +48,6 @@ RESOURCE_TYPE = 'OS::Magnum::Bay'
 class TestMagnumBay(common.HeatTestCase):
     def setUp(self):
         super(TestMagnumBay, self).setUp()
-        self.ctx = utils.dummy_context()
         resource._register_class(RESOURCE_TYPE, bay.Bay)
         t = template_format.parse(magnum_template)
         self.stack = utils.parse_stack(t)
@@ -79,7 +73,7 @@ class TestMagnumBay(common.HeatTestCase):
         self.assertEqual((b.CREATE, b.COMPLETE), b.state)
 
     def test_bay_create_failed(self):
-        cfg.CONF.set_override('action_retry_limit', 0)
+        cfg.CONF.set_override('action_retry_limit', 0, enforce_type=True)
         b = self._create_resource('bay', self.rsrc_defn, self.stack,
                                   stat='CREATE_FAILED')
         exc = self.assertRaises(
@@ -148,6 +142,21 @@ class TestMagnumBay(common.HeatTestCase):
         self.assertEqual((b.DELETE, b.COMPLETE), b.state)
         self.assertEqual(2, self.client.bays.get.call_count)
 
+    def test_bay_get_live_state(self):
+        b = self._create_resource('bay', self.rsrc_defn, self.stack)
+        scheduler.TaskRunner(b.create)()
+        value = mock.MagicMock()
+        value.to_dict.return_value = {
+            'name': 'test_bay',
+            'baymodel': 123456,
+            'node_count': 5,
+            'master_count': 1,
+            'discovery_url': 'https://discovery.etcd.io',
+            'bay_create_timeout': 15}
+        self.client.bays.get.return_value = value
+        reality = b.get_live_state(b.properties)
+        self.assertEqual({'node_count': 5, 'master_count': 1}, reality)
+
 
 class BaymodelConstraintTest(common.HeatTestCase):
     def setUp(self):
