@@ -24,6 +24,7 @@ from heat.common import context
 from heat.common import environment_util as env_util
 from heat.common import exception
 from heat.common import identifier
+from heat.common import policy
 from heat.common import template_format
 from heat.engine.cfn import template as cfntemplate
 from heat.engine import environment
@@ -113,9 +114,6 @@ resources:
 
 
 class StackCreateTest(common.HeatTestCase):
-    def setUp(self):
-        super(StackCreateTest, self).setUp()
-
     def test_wordpress_single_instance_stack_create(self):
         stack = tools.get_stack('test_stack', utils.dummy_context())
         tools.setup_mocks(self.m, stack)
@@ -124,7 +122,7 @@ class StackCreateTest(common.HeatTestCase):
         stack.create()
 
         self.assertIsNotNone(stack['WebServer'])
-        self.assertTrue(int(stack['WebServer'].resource_id) > 0)
+        self.assertGreater(int(stack['WebServer'].resource_id), 0)
         self.assertNotEqual(stack['WebServer'].ipaddress, '0.0.0.0')
 
     def test_wordpress_single_instance_stack_adopt(self):
@@ -190,7 +188,7 @@ class StackCreateTest(common.HeatTestCase):
         self.assertIsNotNone(db_s)
 
         self.assertIsNotNone(stack['WebServer'])
-        self.assertTrue(int(stack['WebServer'].resource_id) > 0)
+        self.assertGreater(int(stack['WebServer'].resource_id), 0)
 
         self.patchobject(fc.servers, 'delete',
                          side_effect=fakes_nova.fake_exception())
@@ -247,7 +245,7 @@ class StackConvergenceServiceCreateUpdateTest(common.HeatTestCase):
                      convergence=True).AndReturn(stack)
 
         self.m.StubOutWithMock(stack, 'validate')
-        stack.validate().AndReturn(None)
+        stack.validate(validate_resources=True).AndReturn(None)
 
         self.m.ReplayAll()
         api_args = {'timeout_mins': 60, 'disable_rollback': False}
@@ -299,7 +297,7 @@ class StackConvergenceServiceCreateUpdateTest(common.HeatTestCase):
                      current_deps=old_stack.current_deps).AndReturn(stack)
 
         self.m.StubOutWithMock(stack, 'validate')
-        stack.validate().AndReturn(None)
+        stack.validate(validate_resources=True).AndReturn(None)
 
         self.m.ReplayAll()
 
@@ -908,10 +906,17 @@ class StackServiceTest(common.HeatTestCase):
             Dummy Func2 Long Description.
             """
 
+        class DummyConditionFunc(object):
+            """Dummy Condition Func.
+
+            Dummy Condition Func Long Description.
+            """
+
         plugin_mock = mock.Mock(
             functions={'dummy1': DummyFunc1,
                        'dummy2': DummyFunc2,
-                       'removed': hot_functions.Removed})
+                       'removed': hot_functions.Removed},
+            condition_functions={'condition_dummy': DummyConditionFunc})
         dummy_tmpl = mock.Mock(plugin=plugin_mock)
 
         class DummyMgr(object):
@@ -927,6 +932,18 @@ class StackServiceTest(common.HeatTestCase):
         self.assertEqual(sorted(expected, key=lambda k: k['functions']),
                          sorted(functions, key=lambda k: k['functions']))
 
+        # test with_condition
+        functions = self.eng.list_template_functions(self.ctx, 'dummytemplate',
+                                                     with_condition=True)
+        expected = [{'functions': 'dummy1',
+                     'description': 'Dummy Func1.'},
+                    {'functions': 'dummy2',
+                     'description': 'Dummy Func2.'},
+                    {'functions': 'condition_dummy',
+                     'description': 'Dummy Condition Func.'}]
+        self.assertEqual(sorted(expected, key=lambda k: k['functions']),
+                         sorted(functions, key=lambda k: k['functions']))
+
     @mock.patch('heat.engine.template._get_template_extension_manager')
     def test_list_template_functions_version_not_found(self, templ_mock):
         class DummyMgr(object):
@@ -1165,6 +1182,7 @@ class StackServiceTest(common.HeatTestCase):
     @mock.patch.object(service.api, 'format_stack_preview', new=mock.Mock())
     @mock.patch.object(service.parser, 'Stack')
     def test_preview_stack_checks_stack_validity(self, mock_parser):
+        self.patchobject(policy.ResourceEnforcer, 'enforce_stack')
         exc = exception.StackValidationFailed(message='Validation Failed')
         mock_parsed_stack = mock.Mock()
         mock_parsed_stack.validate.side_effect = exc
@@ -1322,59 +1340,12 @@ class StackServiceTest(common.HeatTestCase):
                                                 service_check_defer=True,
                                                 resource_validate=False)
         self.assertTrue(lock2.release.called)
+        reason = ('Engine went down during stack %s' % fake_stack.action)
         mock_thread.start_with_acquired_lock.assert_called_once_with(
             fake_stack, lock1,
-            self.eng.set_stack_and_resource_to_failed, fake_stack
+            fake_stack.reset_stack_and_resources_in_progress, reason
         )
 
-    def test_set_stack_and_resource_to_failed(self):
-
-        def fake_stack():
-            stk = mock.MagicMock()
-            stk.action = 'CREATE'
-            stk.id = 'foo'
-            stk.status = 'IN_PROGRESS'
-            stk.FAILED = 'FAILED'
-
-            def mock_stack_state_set(a, s, reason):
-                stk.status = s
-                stk.action = a
-                stk.status_reason = reason
-
-            stk.state_set = mock_stack_state_set
-
-            return stk
-
-        def fake_stack_resource(name, action, status):
-            rs = mock.MagicMock()
-            rs.name = name
-            rs.action = action
-            rs.status = status
-            rs.IN_PROGRESS = 'IN_PROGRESS'
-            rs.FAILED = 'FAILED'
-
-            def mock_resource_state_set(a, s, reason='engine_down'):
-                rs.status = s
-                rs.action = a
-                rs.status_reason = reason
-
-            rs.state_set = mock_resource_state_set
-
-            return rs
-
-        test_stack = fake_stack()
-
-        test_stack.resources = {
-            'r1': fake_stack_resource('r1', 'UPDATE', 'COMPLETE'),
-            'r2': fake_stack_resource('r2', 'UPDATE', 'IN_PROGRESS'),
-            'r3': fake_stack_resource('r3', 'UPDATE', 'FAILED')}
-
-        self.eng.set_stack_and_resource_to_failed(test_stack)
-        self.assertEqual('FAILED', test_stack.status)
-        self.assertEqual('COMPLETE', test_stack.resources.get('r1').status)
-        self.assertEqual('FAILED', test_stack.resources.get('r2').status)
-        self.assertEqual('FAILED', test_stack.resources.get('r3').status)
-
     def test_parse_adopt_stack_data_without_parameters(self):
         cfg.CONF.set_override('enable_stack_adopt', True, enforce_type=True)
         template = {"heat_template_version": "2015-04-30",
