@@ -909,7 +909,7 @@ class ValidateTest(common.HeatTestCase):
         self.ctx = utils.dummy_context()
         self.mock_isa = mock.patch(
             'heat.engine.resource.Resource.is_service_available',
-            return_value=True)
+            return_value=(True, None))
         self.mock_is_service_available = self.mock_isa.start()
         self.addCleanup(self.mock_isa.stop)
         self.engine = service.EngineService('a', 't')
@@ -1425,10 +1425,8 @@ class ValidateTest(common.HeatTestCase):
         t = template_format.parse(test_template_glance_client_exception)
         template = tmpl.Template(t)
         stack = parser.Stack(self.ctx, 'test_stack', template)
-        self.m.StubOutWithMock(self.gc.images, 'get')
-        self.gc.images.get('image_name').AndRaise(glance.exc.HTTPNotFound())
-        self.m.StubOutWithMock(glance.GlanceClientPlugin, '_create')
-        glance.GlanceClientPlugin._create().AndReturn(self.gc)
+        self.m.StubOutWithMock(glance.GlanceClientPlugin, 'client')
+        glance.GlanceClientPlugin.client().AndReturn(self.gc)
         self.stub_FlavorConstraint_validate()
         self.m.ReplayAll()
 
@@ -1622,9 +1620,11 @@ class ValidateTest(common.HeatTestCase):
     def test_validate_invalid_outputs(self):
         t = template_format.parse(test_template_invalid_outputs)
         template = tmpl.Template(t)
+        stack = parser.Stack(self.ctx, 'test_stack', template)
         err = self.assertRaises(exception.StackValidationFailed,
-                                parser.Stack, self.ctx, 'test_stack', template)
-        error_message = ('Arguments to "get_attr" must be of the form '
+                                stack.validate)
+        error_message = ('outputs.string.value.get_attr: Arguments to '
+                         '"get_attr" must be of the form '
                          '[resource_name, attribute, (path), ...]')
         self.assertEqual(error_message, six.text_type(err))
 
@@ -1653,6 +1653,46 @@ class ValidateTest(common.HeatTestCase):
         self.assertEqual('Resource Resource Type type must be string',
                          six.text_type(ex))
 
+    def test_validate_resource_invalid_key(self):
+        t = template_format.parse("""
+        heat_template_version: 2013-05-23
+        resources:
+          resource:
+              type: OS::Heat::TestResource
+              wibble: bar
+        """)
+        template = tmpl.Template(t)
+        stack = parser.Stack(self.ctx, 'test_stack', template)
+        ex = self.assertRaises(exception.StackValidationFailed, stack.validate)
+        self.assertIn('wibble', six.text_type(ex))
+
+    def test_validate_resource_invalid_cfn_key_in_hot(self):
+        t = template_format.parse("""
+        heat_template_version: 2013-05-23
+        resources:
+          resource:
+              type: OS::Heat::TestResource
+              Properties: {foo: bar}
+        """)
+        template = tmpl.Template(t)
+        stack = parser.Stack(self.ctx, 'test_stack', template)
+        ex = self.assertRaises(exception.StackValidationFailed, stack.validate)
+        self.assertIn('Properties', six.text_type(ex))
+
+    def test_validate_resource_invalid_key_cfn(self):
+        t = template_format.parse("""
+        HeatTemplateFormatVersion: '2012-12-12'
+        Resources:
+          Resource:
+            Type: OS::Heat::TestResource
+            Wibble: bar
+        """)
+        template = tmpl.Template(t)
+        stack = parser.Stack(self.ctx, 'test_stack', template)
+        # We have always allowed unknown keys in CFN-style templates, so we
+        # more or less have to keep allowing it.
+        self.assertIsNone(stack.validate())
+
     def test_validate_is_service_available(self):
         t = template_format.parse(
             """
@@ -1662,7 +1702,8 @@ class ValidateTest(common.HeatTestCase):
                 type: AWS::EC2::Instance
             """)
 
-        self.mock_is_service_available.return_value = False
+        self.mock_is_service_available.return_value = (
+            False, 'Service endpoint not in service catalog.')
         ex = self.assertRaises(dispatcher.ExpectedException,
                                self.engine.validate_template,
                                self.ctx,
@@ -1679,7 +1720,8 @@ class ValidateTest(common.HeatTestCase):
                 type: AWS::EC2::Instance
             """)
         engine = service.EngineService('a', 't')
-        self.mock_is_service_available.return_value = False
+        self.mock_is_service_available.return_value = (
+            False, 'Service endpoint not in service catalog.')
 
         res = dict(engine.validate_template(
             self.ctx,
