@@ -22,7 +22,7 @@ from oslo_serialization import jsonutils
 import sqlalchemy
 
 from heat.common import context
-from heat.db import api as db_api
+from heat.db.sqlalchemy import api as db_api
 from heat.db.sqlalchemy import models
 from heat.engine import environment
 from heat.engine import resource
@@ -51,7 +51,9 @@ def random_name():
 
 def setup_dummy_db():
     options.cfg.set_defaults(options.database_opts, sqlite_synchronous=False)
-    options.set_defaults(cfg.CONF, connection="sqlite://", sqlite_db='heat.db')
+    # Uncomment to log SQL
+    # options.cfg.set_defaults(options.database_opts, connection_debug=100)
+    options.set_defaults(cfg.CONF, connection="sqlite://")
     engine = get_engine()
     models.BASE.metadata.create_all(engine)
     engine.connect()
@@ -70,7 +72,7 @@ def reset_dummy_db():
 
 def dummy_context(user='test_username', tenant_id='test_tenant_id',
                   password='password', roles=None, user_id=None,
-                  trust_id=None, region_name=None):
+                  trust_id=None, region_name=None, is_admin=False):
     roles = roles or []
     return context.RequestContext.from_dict({
         'tenant_id': tenant_id,
@@ -79,7 +81,7 @@ def dummy_context(user='test_username', tenant_id='test_tenant_id',
         'user_id': user_id,
         'password': password,
         'roles': roles,
-        'is_admin': False,
+        'is_admin': is_admin,
         'auth_url': 'http://server.test:5000/v2.0',
         'auth_token': 'abcd1234',
         'trust_id': trust_id,
@@ -94,7 +96,7 @@ def parse_stack(t, params=None, files=None, stack_name=None,
     ctx = dummy_context()
     templ = template.Template(t, files=files,
                               env=environment.Environment(params))
-    templ.store()
+    templ.store(ctx)
     if stack_name is None:
         stack_name = random_name()
     stk = stack.Stack(ctx, stack_name, templ, stack_id=stack_id,
